import contextlib
import errno
import os
import socket
import sys

import CoreFoundation
from PyObjCTools.TestSupport import TestCase, min_os_level, onlyIf
import objc

from .test_cfsocket import onTheNetwork


class TestStream(TestCase):
    def testTypes(self):
        try:
            if (
                objc.lookUpClass("__NSCFInputStream")
                is not CoreFoundation.CFReadStreamRef
            ):
                self.assertIsCFType(CoreFoundation.CFReadStreamRef)
        except objc.error:
            try:
                if (
                    objc.lookUpClass("NSCFInputStream")
                    is not CoreFoundation.CFReadStreamRef
                ):
                    self.assertIsCFType(CoreFoundation.CFReadStreamRef)
            except objc.error:
                self.assertIsCFType(CoreFoundation.CFReadStreamRef)

        try:
            if (
                objc.lookUpClass("__NSCFOutputStream")
                is not CoreFoundation.CFWriteStreamRef
            ):
                self.assertIsCFType(CoreFoundation.CFWriteStreamRef)
        except objc.error:
            try:
                if (
                    objc.lookUpClass("NSCFOutputStream")
                    is not CoreFoundation.CFWriteStreamRef
                ):
                    self.assertIsCFType(CoreFoundation.CFWriteStreamRef)
            except objc.error:
                self.assertIsCFType(CoreFoundation.CFWriteStreamRef)

    def testConstants(self):
        self.assertEqual(CoreFoundation.kCFStreamStatusNotOpen, 0)
        self.assertEqual(CoreFoundation.kCFStreamStatusOpening, 1)
        self.assertEqual(CoreFoundation.kCFStreamStatusOpen, 2)
        self.assertEqual(CoreFoundation.kCFStreamStatusReading, 3)
        self.assertEqual(CoreFoundation.kCFStreamStatusWriting, 4)
        self.assertEqual(CoreFoundation.kCFStreamStatusAtEnd, 5)
        self.assertEqual(CoreFoundation.kCFStreamStatusClosed, 6)
        self.assertEqual(CoreFoundation.kCFStreamStatusError, 7)
        self.assertEqual(CoreFoundation.kCFStreamEventNone, 0)
        self.assertEqual(CoreFoundation.kCFStreamEventOpenCompleted, 1)
        self.assertEqual(CoreFoundation.kCFStreamEventHasBytesAvailable, 2)
        self.assertEqual(CoreFoundation.kCFStreamEventCanAcceptBytes, 4)
        self.assertEqual(CoreFoundation.kCFStreamEventErrorOccurred, 8)
        self.assertEqual(CoreFoundation.kCFStreamEventEndEncountered, 16)
        self.assertEqual(CoreFoundation.kCFStreamErrorDomainCustom, -1)
        self.assertEqual(CoreFoundation.kCFStreamErrorDomainPOSIX, 1)
        self.assertEqual(CoreFoundation.kCFStreamErrorDomainMacOSStatus, 2)
        self.assertIsInstance(CoreFoundation.kCFStreamPropertyDataWritten, str)
        self.assertIsInstance(CoreFoundation.kCFStreamPropertyAppendToFile, str)
        self.assertIsInstance(CoreFoundation.kCFStreamPropertyFileCurrentOffset, str)
        self.assertIsInstance(CoreFoundation.kCFStreamPropertySocketNativeHandle, str)
        self.assertIsInstance(CoreFoundation.kCFStreamPropertySocketRemoteHostName, str)
        self.assertIsInstance(
            CoreFoundation.kCFStreamPropertySocketRemotePortNumber, str
        )

        self.assertIsInstance(CoreFoundation.kCFStreamErrorDomainSOCKS, int)
        self.assertIsInstance(CoreFoundation.kCFStreamPropertySOCKSProxy, str)
        self.assertIsInstance(CoreFoundation.kCFStreamPropertySOCKSProxyHost, str)
        self.assertIsInstance(CoreFoundation.kCFStreamPropertySOCKSProxyPort, str)
        self.assertIsInstance(CoreFoundation.kCFStreamPropertySOCKSVersion, str)
        self.assertIsInstance(CoreFoundation.kCFStreamSocketSOCKSVersion4, str)
        self.assertIsInstance(CoreFoundation.kCFStreamSocketSOCKSVersion5, str)
        self.assertIsInstance(CoreFoundation.kCFStreamPropertySOCKSUser, str)
        self.assertIsInstance(CoreFoundation.kCFStreamPropertySOCKSPassword, str)
        self.assertIsInstance(CoreFoundation.kCFStreamErrorDomainSSL, int)
        self.assertIsInstance(CoreFoundation.kCFStreamPropertySocketSecurityLevel, str)
        self.assertIsInstance(CoreFoundation.kCFStreamSocketSecurityLevelNone, str)
        self.assertIsInstance(CoreFoundation.kCFStreamSocketSecurityLevelSSLv2, str)
        self.assertIsInstance(CoreFoundation.kCFStreamSocketSecurityLevelSSLv3, str)
        self.assertIsInstance(CoreFoundation.kCFStreamSocketSecurityLevelTLSv1, str)
        self.assertIsInstance(
            CoreFoundation.kCFStreamSocketSecurityLevelNegotiatedSSL, str
        )
        self.assertIsInstance(
            CoreFoundation.kCFStreamPropertyShouldCloseNativeSocket, str
        )

    def testStructs(self):
        o = CoreFoundation.CFStreamError()
        self.assertHasAttr(o, "domain")
        self.assertHasAttr(o, "error")

    def testGetTypeID(self):
        v = CoreFoundation.CFReadStreamGetTypeID()
        self.assertIsInstance(v, int)
        v = CoreFoundation.CFWriteStreamGetTypeID()
        self.assertIsInstance(v, int)

    def testReadStream(self):
        strval = b"hello world"
        self.assertArgHasType(
            CoreFoundation.CFReadStreamCreateWithBytesNoCopy, 1, b"n^v"
        )
        self.assertArgSizeInArg(CoreFoundation.CFReadStreamCreateWithBytesNoCopy, 1, 2)
        stream = CoreFoundation.CFReadStreamCreateWithBytesNoCopy(
            None, strval, len(strval), CoreFoundation.kCFAllocatorNull
        )
        self.assertIsInstance(stream, CoreFoundation.CFReadStreamRef)
        r, buf = CoreFoundation.CFReadStreamRead(stream, None, 10)
        self.assertEqual(r, -1)
        self.assertEqual(buf, b"")

        self.assertResultIsCFRetained(CoreFoundation.CFReadStreamCopyError)
        err = CoreFoundation.CFReadStreamCopyError(stream)
        if err is not None:
            self.assertIsInstance(err, CoreFoundation.CFErrorRef)
        status = CoreFoundation.CFReadStreamGetStatus(stream)
        self.assertIsInstance(status, int)
        self.assertEqual(status, CoreFoundation.kCFStreamStatusNotOpen)

        self.assertResultIsBOOL(CoreFoundation.CFReadStreamOpen)
        r = CoreFoundation.CFReadStreamOpen(stream)
        self.assertIs(r, True)
        status = CoreFoundation.CFReadStreamGetStatus(stream)
        self.assertIsInstance(status, int)
        self.assertEqual(status, CoreFoundation.kCFStreamStatusOpen)

        self.assertResultIsBOOL(CoreFoundation.CFReadStreamHasBytesAvailable)
        r = CoreFoundation.CFReadStreamHasBytesAvailable(stream)
        self.assertIs(r, True)
        self.assertArgHasType(CoreFoundation.CFReadStreamRead, 1, b"o^v")
        self.assertArgSizeInArg(CoreFoundation.CFReadStreamRead, 1, 2)
        self.assertArgSizeInResult(CoreFoundation.CFReadStreamRead, 1)
        r, buf = CoreFoundation.CFReadStreamRead(stream, None, 5)
        self.assertEqual(r, 5)
        self.assertEqual(buf, b"hello")

        r, buf = CoreFoundation.CFReadStreamRead(stream, None, 10)
        self.assertEqual(r, 6)
        self.assertEqual(buf, b" world")

        r = CoreFoundation.CFReadStreamHasBytesAvailable(stream)
        self.assertIs(r, False)
        r = CoreFoundation.CFReadStreamClose(stream)
        self.assertIs(r, None)
        status = CoreFoundation.CFReadStreamGetStatus(stream)
        self.assertIsInstance(status, int)
        self.assertEqual(status, CoreFoundation.kCFStreamStatusClosed)

        del stream

        self.assertResultIsCFRetained(CoreFoundation.CFReadStreamCreateWithFile)
        stream = CoreFoundation.CFReadStreamCreateWithFile(
            None, CoreFoundation.CFURLCreateWithString(None, "file:///etc/shells", None)
        )
        self.assertIsInstance(stream, CoreFoundation.CFReadStreamRef)
        r = CoreFoundation.CFReadStreamOpen(stream)
        self.assertIs(r, True)
        status = CoreFoundation.CFReadStreamGetStatus(stream)
        self.assertIsInstance(status, int)
        self.assertEqual(status, CoreFoundation.kCFStreamStatusOpen)

        r, buf = CoreFoundation.CFReadStreamRead(stream, None, 5)
        self.assertEqual(r, 5)
        self.assertIsInstance(buf, bytes)
        self.assertResultSizeInArg(CoreFoundation.CFReadStreamGetBuffer, 2)
        self.assertResultHasType(CoreFoundation.CFReadStreamGetBuffer, b"^v")
        self.assertArgIsOut(CoreFoundation.CFReadStreamGetBuffer, 2)
        buf, numBytes = CoreFoundation.CFReadStreamGetBuffer(stream, 20, None)
        if buf is objc.NULL:
            self.assertEqual(numBytes, 0)
        else:
            self.assertIsInstance(buf, str)
            self.assertEqual(numBytes, len(buf))

        val = CoreFoundation.CFReadStreamCopyProperty(
            stream, CoreFoundation.kCFStreamPropertyFileCurrentOffset
        )
        self.assertEqual(val, 5)

        r = CoreFoundation.CFReadStreamSetProperty(
            stream, CoreFoundation.kCFStreamPropertyFileCurrentOffset, 10
        )
        self.assertIs(r, True)
        val = CoreFoundation.CFReadStreamCopyProperty(
            stream, CoreFoundation.kCFStreamPropertyFileCurrentOffset
        )
        self.assertEqual(val, 10)

        err = CoreFoundation.CFReadStreamGetError(stream)
        self.assertIsInstance(err, CoreFoundation.CFStreamError)
        self.assertEqual(err.domain, 0)
        self.assertEqual(err.error, 0)

    def testWriteStream(self):
        import array

        a = array.array("b", b" " * 20)

        # XXX: cannot express the actual type as metadata :-(
        self.assertArgHasType(CoreFoundation.CFWriteStreamCreateWithBuffer, 1, b"n^v")
        self.assertArgSizeInArg(CoreFoundation.CFWriteStreamCreateWithBuffer, 1, 2)
        stream = CoreFoundation.CFWriteStreamCreateWithBuffer(None, a, 20)
        self.assertIsInstance(stream, CoreFoundation.CFWriteStreamRef)
        self.assertResultIsBOOL(CoreFoundation.CFWriteStreamOpen)
        r = CoreFoundation.CFWriteStreamOpen(stream)
        self.assertIs(r, True)
        status = CoreFoundation.CFWriteStreamGetStatus(stream)
        self.assertIsInstance(status, int)
        self.assertEqual(status, CoreFoundation.kCFStreamStatusOpen)

        self.assertResultIsBOOL(CoreFoundation.CFWriteStreamCanAcceptBytes)
        b = CoreFoundation.CFWriteStreamCanAcceptBytes(stream)
        self.assertIs(b, True)
        self.assertArgHasType(CoreFoundation.CFWriteStreamWrite, 1, b"n^v")
        self.assertArgSizeInArg(CoreFoundation.CFWriteStreamWrite, 1, 2)
        n = CoreFoundation.CFWriteStreamWrite(stream, b"0123456789ABCDE", 15)
        self.assertEqual(n, 15)

        if sys.version_info[0] == 3:
            self.assertEqual(bytes(a[0:1]), b"0")
            self.assertEqual(bytes(a[1:2]), b"1")
            self.assertEqual(bytes(a[9:10]), b"9")
        else:
            self.assertEqual((a[0]), ord("0"))
            self.assertEqual((a[1]), ord("1"))
            self.assertEqual((a[9]), ord("9"))

        n = CoreFoundation.CFWriteStreamWrite(stream, b"0123456789ABCDE", 15)
        self.assertEqual(n, -1)

        err = CoreFoundation.CFWriteStreamCopyError(stream)
        self.assertIsInstance(err, CoreFoundation.CFErrorRef)
        err = CoreFoundation.CFWriteStreamGetError(stream)
        self.assertIsInstance(err, CoreFoundation.CFStreamError)
        self.assertEqual(err.domain, CoreFoundation.kCFStreamErrorDomainPOSIX)
        self.assertEqual(err.error, errno.ENOMEM)
        status = CoreFoundation.CFWriteStreamGetStatus(stream)
        self.assertIsInstance(status, int)
        self.assertEqual(status, CoreFoundation.kCFStreamStatusError)

        del stream

        self.assertResultIsCFRetained(
            CoreFoundation.CFWriteStreamCreateWithAllocatedBuffers
        )
        stream = CoreFoundation.CFWriteStreamCreateWithAllocatedBuffers(None, None)
        self.assertIsInstance(stream, CoreFoundation.CFWriteStreamRef)
        r = CoreFoundation.CFWriteStreamOpen(stream)
        self.assertIs(r, True)
        n = CoreFoundation.CFWriteStreamWrite(stream, b"0123456789ABCDE", 15)
        self.assertEqual(n, 15)

        self.assertResultIsCFRetained(CoreFoundation.CFWriteStreamCopyProperty)
        buf = CoreFoundation.CFWriteStreamCopyProperty(
            stream, CoreFoundation.kCFStreamPropertyDataWritten
        )
        self.assertIsInstance(buf, CoreFoundation.CFDataRef)
        buf = CoreFoundation.CFDataGetBytes(
            buf, (0, CoreFoundation.CFDataGetLength(buf)), None
        )
        self.assertIsInstance(buf, bytes)
        self.assertEqual(buf, b"0123456789ABCDE")

        CoreFoundation.CFWriteStreamClose(stream)
        status = CoreFoundation.CFWriteStreamGetStatus(stream)
        self.assertIsInstance(status, int)
        self.assertEqual(status, CoreFoundation.kCFStreamStatusClosed)

        del stream

        stream = CoreFoundation.CFWriteStreamCreateWithFile(
            None,
            CoreFoundation.CFURLCreateWithString(
                None, "file:///tmp/pyobjc.test.txt", None
            ),
        )
        self.assertIsInstance(stream, CoreFoundation.CFWriteStreamRef)
        r = CoreFoundation.CFWriteStreamOpen(stream)
        self.assertIs(r, True)
        n = CoreFoundation.CFWriteStreamWrite(stream, b"0123456789ABCDE", 15)
        self.assertEqual(n, 15)

        self.assertResultIsCFRetained(CoreFoundation.CFReadStreamCopyProperty)
        val = CoreFoundation.CFReadStreamCopyProperty(
            stream, CoreFoundation.kCFStreamPropertyFileCurrentOffset
        )
        self.assertEqual(val, 15)

        self.assertResultIsBOOL(CoreFoundation.CFReadStreamSetProperty)
        r = CoreFoundation.CFReadStreamSetProperty(
            stream, CoreFoundation.kCFStreamPropertyFileCurrentOffset, 10
        )
        self.assertIs(r, True)
        val = CoreFoundation.CFReadStreamCopyProperty(
            stream, CoreFoundation.kCFStreamPropertyFileCurrentOffset
        )
        self.assertEqual(val, 10)

        CoreFoundation.CFWriteStreamClose(stream)
        status = CoreFoundation.CFWriteStreamGetStatus(stream)
        self.assertIsInstance(status, int)
        self.assertEqual(status, CoreFoundation.kCFStreamStatusClosed)

        self.assertResultIsBOOL(CoreFoundation.CFWriteStreamSetProperty)
        CoreFoundation.CFWriteStreamSetProperty(
            stream, CoreFoundation.kCFStreamPropertyFileCurrentOffset, 0
        )

        with open("/tmp/pyobjc.test.txt", "rb") as fp:
            data = fp.read()
        self.assertEqual(data, b"0123456789ABCDE")
        os.unlink("/tmp/pyobjc.test.txt")

    def testStreamPair(self):

        self.assertArgIsOut(CoreFoundation.CFStreamCreateBoundPair, 1)
        self.assertArgIsOut(CoreFoundation.CFStreamCreateBoundPair, 2)
        readStream, writeStream = CoreFoundation.CFStreamCreateBoundPair(
            None, None, None, 1024 * 1024
        )
        self.assertIsInstance(readStream, CoreFoundation.CFReadStreamRef)
        self.assertIsInstance(writeStream, CoreFoundation.CFWriteStreamRef)
        # Make sure we actually have streams instead of random pointers.
        status = CoreFoundation.CFReadStreamGetStatus(readStream)
        self.assertIsInstance(status, int)
        self.assertEqual(status, CoreFoundation.kCFStreamStatusNotOpen)

        status = CoreFoundation.CFWriteStreamGetStatus(writeStream)
        self.assertIsInstance(status, int)
        self.assertEqual(status, CoreFoundation.kCFStreamStatusNotOpen)

        del readStream, writeStream

    @onlyIf(onTheNetwork(), "Test requires a working Internet connection")
    def testSockets(self):
        with contextlib.closing(
            socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        ) as sd:
            sd.connect(("www.apple.com", 80))

            self.assertArgIsOut(CoreFoundation.CFStreamCreatePairWithSocket, 2)
            self.assertArgIsOut(CoreFoundation.CFStreamCreatePairWithSocket, 3)
            readStream, writeStream = CoreFoundation.CFStreamCreatePairWithSocket(
                None, sd.fileno(), None, None
            )

            status = CoreFoundation.CFReadStreamGetStatus(readStream)
            self.assertIsInstance(status, int)
            self.assertEqual(status, CoreFoundation.kCFStreamStatusNotOpen)

            status = CoreFoundation.CFWriteStreamGetStatus(writeStream)
            self.assertIsInstance(status, int)
            self.assertEqual(status, CoreFoundation.kCFStreamStatusNotOpen)

            del readStream, writeStream, sd

        self.assertArgIsOut(CoreFoundation.CFStreamCreatePairWithSocketToHost, 3)
        self.assertArgIsOut(CoreFoundation.CFStreamCreatePairWithSocketToHost, 4)
        readStream, writeStream = CoreFoundation.CFStreamCreatePairWithSocketToHost(
            None, "www.apple.com", 80, None, None
        )

        status = CoreFoundation.CFReadStreamGetStatus(readStream)
        self.assertIsInstance(status, int)
        self.assertEqual(status, CoreFoundation.kCFStreamStatusNotOpen)

        status = CoreFoundation.CFWriteStreamGetStatus(writeStream)
        self.assertIsInstance(status, int)
        self.assertEqual(status, CoreFoundation.kCFStreamStatusNotOpen)

        del readStream, writeStream

        # Note: I don't expect anyone to actually use this api, building
        # struct sockaddr buffers by hand is madness in python.
        ip = socket.gethostbyname("www.apple.com")
        ip = map(int, ip.split("."))

        import struct

        sockaddr = struct.pack(">BBHBBBB", 16, socket.AF_INET, 80, *ip)

        signature = CoreFoundation.CFSocketSignature(
            protocolFamily=socket.AF_INET,
            socketType=socket.SOCK_STREAM,
            protocol=0,
            address=sockaddr,
        )

        self.assertArgIsOut(CoreFoundation.CFStreamCreatePairWithPeerSocketSignature, 2)
        self.assertArgIsOut(CoreFoundation.CFStreamCreatePairWithPeerSocketSignature, 3)
        (
            readStream,
            writeStream,
        ) = CoreFoundation.CFStreamCreatePairWithPeerSocketSignature(
            None, signature, None, None
        )

        self.assertResultIsCFRetained(CoreFoundation.CFWriteStreamCopyError)
        status = CoreFoundation.CFReadStreamGetStatus(readStream)
        self.assertIsInstance(status, int)
        self.assertEqual(status, CoreFoundation.kCFStreamStatusNotOpen)

        status = CoreFoundation.CFWriteStreamGetStatus(writeStream)
        self.assertIsInstance(status, int)
        self.assertEqual(status, CoreFoundation.kCFStreamStatusNotOpen)

    def testReadSocketASync(self):
        rl = CoreFoundation.CFRunLoopGetCurrent()

        strval = b"hello world"
        readStream = CoreFoundation.CFReadStreamCreateWithBytesNoCopy(
            None, strval, len(strval), CoreFoundation.kCFAllocatorNull
        )
        self.assertIsInstance(readStream, CoreFoundation.CFReadStreamRef)
        data = {}

        state = []

        def callback(stream, kind, info):
            state.append((stream, kind, info))

        status = CoreFoundation.CFReadStreamGetStatus(readStream)
        self.assertIsInstance(status, int)
        self.assertEqual(status, CoreFoundation.kCFStreamStatusNotOpen)

        CoreFoundation.CFReadStreamOpen(readStream)

        ok = CoreFoundation.CFReadStreamSetClient(
            readStream,
            CoreFoundation.kCFStreamEventHasBytesAvailable
            | CoreFoundation.kCFStreamEventErrorOccurred
            | CoreFoundation.kCFStreamEventEndEncountered,
            callback,
            data,
        )
        self.assertTrue(ok)

        CoreFoundation.CFReadStreamScheduleWithRunLoop(
            readStream, rl, CoreFoundation.kCFRunLoopDefaultMode
        )
        try:
            CoreFoundation.CFRunLoopRunInMode(
                CoreFoundation.kCFRunLoopDefaultMode, 1.0, True
            )
            CoreFoundation.CFRunLoopWakeUp(rl)
        finally:
            CoreFoundation.CFReadStreamClose(readStream)
            CoreFoundation.CFReadStreamUnscheduleFromRunLoop(
                readStream, rl, CoreFoundation.kCFRunLoopDefaultMode
            )
            ok = CoreFoundation.CFReadStreamSetClient(
                readStream,
                CoreFoundation.kCFStreamEventHasBytesAvailable
                | CoreFoundation.kCFStreamEventErrorOccurred
                | CoreFoundation.kCFStreamEventEndEncountered,
                callback,
                objc.NULL,
            )
            self.assertTrue(ok)

        self.assertEqual(len(state), 1)
        self.assertIs(state[0][0], readStream)
        self.assertIs(state[0][2], data)
        self.assertEqual(state[0][1], CoreFoundation.kCFStreamEventHasBytesAvailable)

    def testWriteSocketAsync(self):
        rl = CoreFoundation.CFRunLoopGetCurrent()

        import array

        a = array.array("b", b" " * 20)

        writeStream = CoreFoundation.CFWriteStreamCreateWithBuffer(None, a, 20)
        self.assertIsInstance(writeStream, CoreFoundation.CFWriteStreamRef)
        r = CoreFoundation.CFWriteStreamOpen(writeStream)
        self.assertIs(r, True)
        data = {}
        state = []

        def callback(stream, kind, info):
            state.append((stream, kind, info))

        ok = CoreFoundation.CFWriteStreamSetClient(
            writeStream,
            CoreFoundation.kCFStreamEventCanAcceptBytes
            | CoreFoundation.kCFStreamEventErrorOccurred
            | CoreFoundation.kCFStreamEventEndEncountered,
            callback,
            data,
        )
        self.assertTrue(ok)

        CoreFoundation.CFWriteStreamScheduleWithRunLoop(
            writeStream, rl, CoreFoundation.kCFRunLoopDefaultMode
        )
        try:
            CoreFoundation.CFRunLoopRunInMode(
                CoreFoundation.kCFRunLoopDefaultMode, 1.0, True
            )
            CoreFoundation.CFRunLoopWakeUp(rl)
        finally:
            CoreFoundation.CFWriteStreamClose(writeStream)
            CoreFoundation.CFWriteStreamUnscheduleFromRunLoop(
                writeStream, rl, CoreFoundation.kCFRunLoopDefaultMode
            )
            ok = CoreFoundation.CFWriteStreamSetClient(
                writeStream,
                CoreFoundation.kCFStreamEventCanAcceptBytes
                | CoreFoundation.kCFStreamEventErrorOccurred
                | CoreFoundation.kCFStreamEventEndEncountered,
                callback,
                objc.NULL,
            )
            self.assertTrue(ok)

        self.assertEqual(len(state), 1)
        self.assertIs(state[0][0], writeStream)
        self.assertIs(state[0][2], data)
        self.assertEqual(state[0][1], CoreFoundation.kCFStreamEventCanAcceptBytes)

    @min_os_level("10.9")
    def testFunctions10_9(self):
        CoreFoundation.CFReadStreamSetDispatchQueue  # Nothing to test
        CoreFoundation.CFWriteStreamSetDispatchQueue  # Nothing to test
        CoreFoundation.CFReadStreamCopyDispatchQueue  # Nothing to test
        CoreFoundation.CFWriteStreamCopyDispatchQueue  # Nothing to test



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSDebug(TestCase):
    def testFunctions(self):
        self.assertResultIsBOOL(Foundation.NSIsFreedObject)
        Foundation.NSRecordAllocationEvent
        Foundation.NSFrameAddress
        Foundation.NSReturnAddress
        Foundation.NSCountFrames
        Foundation.NSRecordAllocationEvent

    def testConstants(self):
        self.assertEqual(Foundation.NSObjectAutoreleasedEvent, 3)
        self.assertEqual(Foundation.NSObjectExtraRefIncrementedEvent, 4)
        self.assertEqual(Foundation.NSObjectExtraRefDecrementedEvent, 5)
        self.assertEqual(Foundation.NSObjectInternalRefIncrementedEvent, 6)
        self.assertEqual(Foundation.NSObjectInternalRefDecrementedEvent, 7)

        self.assertIsInstance(Foundation.NSDebugEnabled, bool)
        self.assertIsInstance(Foundation.NSZombieEnabled, bool)
        self.assertIsInstance(Foundation.NSDeallocateZombies, bool)
        self.assertIsInstance(Foundation.NSHangOnUncaughtException, bool)
        self.assertIsInstance(Foundation.NSKeepAllocationStatistics, bool)



import AppDelegate  # noqa: F401
import MyBaseGradientView  # noqa: F401
import MyBezierGradientView  # noqa: F401
import MyRectGradientView  # noqa: F401
import MyWindowController  # noqa: F401
from PyObjCTools import AppHelper

AppHelper.runEventLoop()



import Foundation
from PyObjCTools.TestSupport import TestCase, min_sdk_level
import objc


class PortDelegate(Foundation.NSObject):
    def handleMachMessage_(self, m):
        pass


class NSPortHelper(Foundation.NSPort):
    def sendBeforeDate_components_from_reserved_(self, a, b, c, d):
        pass

    def sendBeforeDate_msgid_components_from_reserved_(self, a, b, c, d, e):
        pass


class TestNSPort(TestCase):
    def testConstants(self):
        self.assertIsInstance(Foundation.NSPortDidBecomeInvalidNotification, str)
        self.assertEqual(Foundation.NSMachPortDeallocateNone, 0)
        self.assertEqual(Foundation.NSMachPortDeallocateSendRight, (1 << 0))
        self.assertEqual(Foundation.NSMachPortDeallocateReceiveRight, (1 << 1))

    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSPort.isValid)

        self.assertResultIsBOOL(
            Foundation.NSPortHelper.sendBeforeDate_components_from_reserved_
        )
        self.assertResultIsBOOL(
            Foundation.NSPortHelper.sendBeforeDate_msgid_components_from_reserved_
        )

        self.assertArgHasType(
            Foundation.NSPortHelper.sendBeforeDate_components_from_reserved_,
            3,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            Foundation.NSPortHelper.sendBeforeDate_msgid_components_from_reserved_,
            4,
            objc._C_NSUInteger,
        )

        self.assertArgHasType(PortDelegate.handleMachMessage_, 0, b"^v")

    @min_sdk_level("10.6")
    def testProtocols(self):
        objc.protocolNamed("NSPortDelegate")
        objc.protocolNamed("NSMachPortDelegate")



#
#  __main__.py
#  GraphicsBindings
#
#  Created by Fred Flintstone on 11.02.05.
#  Copyright (c) 2005 __MyCompanyName__. All rights reserved.
#

import Circle  # noqa: F401
import GraphicsArrayController  # noqa: F401
import GraphicsBindingsDocument  # noqa: F401
import GraphicsView  # noqa: F401
import JoystickView  # noqa: F401

# start the event loop
import objc
from PyObjCTools import AppHelper

objc.setVerbose(1)
AppHelper.runEventLoop(argv=[])



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestNSURL(TestCase):
    def testMethods(self):
        self.assertArgIsBOOL(Foundation.NSURL.initFileURLWithPath_isDirectory_, 1)
        self.assertArgIsBOOL(Foundation.NSURL.fileURLWithPath_isDirectory_, 1)
        self.assertResultIsBOOL(Foundation.NSURL.isFileURL)

        self.assertArgIsBOOL(Foundation.NSURL.resourceDataUsingCache_, 0)
        self.assertArgIsBOOL(
            Foundation.NSURL.loadResourceDataNotifyingClient_usingCache_, 1
        )
        self.assertResultIsBOOL(Foundation.NSURL.setResourceData_)
        self.assertResultIsBOOL(Foundation.NSURL.setProperty_forKey_)
        self.assertArgIsBOOL(Foundation.NSURL.URLHandleUsingCache_, 0)

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBOOL(Foundation.NSURL.startAccessingSecurityScopedResource)

    @min_os_level("10.9")
    def testMethods10_9(self):
        self.assertArgIsIn(
            Foundation.NSURL.initFileURLWithFileSystemRepresentation_isDirectory_relativeToURL_,
            0,
        )
        self.assertArgIsBOOL(
            Foundation.NSURL.initFileURLWithFileSystemRepresentation_isDirectory_relativeToURL_,
            1,
        )

        self.assertArgIsIn(
            Foundation.NSURL.fileURLWithFileSystemRepresentation_isDirectory_relativeToURL_,
            0,
        )
        self.assertArgIsBOOL(
            Foundation.NSURL.fileURLWithFileSystemRepresentation_isDirectory_relativeToURL_,
            1,
        )

        self.assertResultIsBOOL(Foundation.NSURL.getFileSystemRepresentation_maxLength_)
        self.assertArgIsOut(Foundation.NSURL.getFileSystemRepresentation_maxLength_, 0)
        self.assertArgSizeInArg(
            Foundation.NSURL.getFileSystemRepresentation_maxLength_, 0, 1
        )

        self.assertArgIsBOOL(
            Foundation.NSURLComponents.initWithURL_resolvingAgainstBaseURL_, 1
        )
        self.assertArgIsBOOL(
            Foundation.NSURLComponents.componentsWithURL_resolvingAgainstBaseURL_, 1
        )

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertArgIsOut(
            Foundation.NSURL.URLByResolvingAliasFileAtURL_options_error_, 2
        )

        self.assertResultIsBOOL(
            Foundation.NSURL.getPromisedItemResourceValue_forKey_error_
        )
        self.assertArgIsOut(
            Foundation.NSURL.getPromisedItemResourceValue_forKey_error_, 2
        )
        self.assertArgIsOut(
            Foundation.NSURL.promisedItemResourceValuesForKeys_error_, 1
        )

        self.assertResultIsBOOL(
            Foundation.NSURL.checkPromisedItemIsReachableAndReturnError_
        )
        self.assertArgIsOut(
            Foundation.NSURL.checkPromisedItemIsReachableAndReturnError_, 0
        )

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertArgIsBOOL(
            Foundation.NSURL.initFileURLWithPath_isDirectory_relativeToURL_, 1
        )
        self.assertArgIsBOOL(
            Foundation.NSURL.fileURLWithPath_isDirectory_relativeToURL_, 1
        )
        self.assertResultIsBOOL(Foundation.NSURL.hasDirectoryPath)

    def testConstants(self):
        self.assertIsInstance(Foundation.NSURLFileScheme, str)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(
            Foundation.NSURLBookmarkCreationPreferFileIDResolution, (1 << 8)
        )
        self.assertEqual(Foundation.NSURLBookmarkCreationMinimalBookmark, (1 << 9))
        self.assertEqual(
            Foundation.NSURLBookmarkCreationSuitableForBookmarkFile, (1 << 10)
        )
        self.assertEqual(Foundation.NSURLBookmarkResolutionWithoutUI, (1 << 8))
        self.assertEqual(Foundation.NSURLBookmarkResolutionWithoutMounting, (1 << 9))

        self.assertIsInstance(Foundation.NSURLNameKey, str)
        self.assertIsInstance(Foundation.NSURLLocalizedNameKey, str)
        self.assertIsInstance(Foundation.NSURLIsRegularFileKey, str)
        self.assertIsInstance(Foundation.NSURLIsDirectoryKey, str)
        self.assertIsInstance(Foundation.NSURLIsSymbolicLinkKey, str)
        self.assertIsInstance(Foundation.NSURLIsVolumeKey, str)
        self.assertIsInstance(Foundation.NSURLIsPackageKey, str)
        self.assertIsInstance(Foundation.NSURLIsSystemImmutableKey, str)
        self.assertIsInstance(Foundation.NSURLIsUserImmutableKey, str)
        self.assertIsInstance(Foundation.NSURLIsHiddenKey, str)
        self.assertIsInstance(Foundation.NSURLHasHiddenExtensionKey, str)
        self.assertIsInstance(Foundation.NSURLCreationDateKey, str)
        self.assertIsInstance(Foundation.NSURLContentAccessDateKey, str)
        self.assertIsInstance(Foundation.NSURLContentModificationDateKey, str)
        self.assertIsInstance(Foundation.NSURLAttributeModificationDateKey, str)
        self.assertIsInstance(Foundation.NSURLLinkCountKey, str)
        self.assertIsInstance(Foundation.NSURLParentDirectoryURLKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeURLKey, str)
        self.assertIsInstance(Foundation.NSURLTypeIdentifierKey, str)
        self.assertIsInstance(Foundation.NSURLLocalizedTypeDescriptionKey, str)
        self.assertIsInstance(Foundation.NSURLLabelNumberKey, str)
        self.assertIsInstance(Foundation.NSURLLabelColorKey, str)
        self.assertIsInstance(Foundation.NSURLLocalizedLabelKey, str)
        self.assertIsInstance(Foundation.NSURLEffectiveIconKey, str)
        self.assertIsInstance(Foundation.NSURLCustomIconKey, str)
        self.assertIsInstance(Foundation.NSURLFileSizeKey, str)
        self.assertIsInstance(Foundation.NSURLFileAllocatedSizeKey, str)
        self.assertIsInstance(Foundation.NSURLIsAliasFileKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeLocalizedFormatDescriptionKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeTotalCapacityKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeAvailableCapacityKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeResourceCountKey, str)

    @min_os_level("10.6")
    def testConstants10_6_2(self):
        self.assertIsInstance(Foundation.NSURLVolumeSupportsPersistentIDsKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeSupportsSymbolicLinksKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeSupportsHardLinksKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeSupportsJournalingKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeIsJournalingKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeSupportsSparseFilesKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeSupportsZeroRunsKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeSupportsCaseSensitiveNamesKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeSupportsCasePreservedNamesKey, str)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(Foundation.NSURLBookmarkCreationWithSecurityScope, (1 << 11))
        self.assertEqual(
            Foundation.NSURLBookmarkCreationSecurityScopeAllowOnlyReadAccess, (1 << 12)
        )
        self.assertEqual(Foundation.NSURLBookmarkResolutionWithSecurityScope, (1 << 10))
        self.assertEqual(
            Foundation.NSURLBookmarkCreationWithoutImplicitSecurityScope, (1 << 29)
        )

        self.assertIsInstance(Foundation.NSURLKeysOfUnsetValuesKey, str)

        self.assertIsInstance(Foundation.NSURLFileResourceIdentifierKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeIdentifierKey, str)
        self.assertIsInstance(Foundation.NSURLPreferredIOBlockSizeKey, str)
        self.assertIsInstance(Foundation.NSURLIsReadableKey, str)
        self.assertIsInstance(Foundation.NSURLIsWritableKey, str)
        self.assertIsInstance(Foundation.NSURLIsExecutableKey, str)
        self.assertIsInstance(Foundation.NSURLIsMountTriggerKey, str)
        self.assertIsInstance(Foundation.NSURLFileSecurityKey, str)
        self.assertIsInstance(Foundation.NSURLFileResourceTypeKey, str)
        self.assertIsInstance(Foundation.NSURLFileResourceTypeNamedPipe, str)
        self.assertIsInstance(Foundation.NSURLFileResourceTypeCharacterSpecial, str)
        self.assertIsInstance(Foundation.NSURLFileResourceTypeDirectory, str)
        self.assertIsInstance(Foundation.NSURLFileResourceTypeBlockSpecial, str)
        self.assertIsInstance(Foundation.NSURLFileResourceTypeRegular, str)
        self.assertIsInstance(Foundation.NSURLFileResourceTypeSymbolicLink, str)
        self.assertIsInstance(Foundation.NSURLFileResourceTypeSocket, str)
        self.assertIsInstance(Foundation.NSURLFileResourceTypeUnknown, str)
        self.assertIsInstance(Foundation.NSURLTotalFileSizeKey, str)
        self.assertIsInstance(Foundation.NSURLTotalFileAllocatedSizeKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeSupportsRootDirectoryDatesKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeSupportsVolumeSizesKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeSupportsRenamingKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeSupportsAdvisoryFileLockingKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeSupportsExtendedSecurityKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeIsBrowsableKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeMaximumFileSizeKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeIsEjectableKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeIsRemovableKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeIsInternalKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeIsAutomountedKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeIsLocalKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeIsReadOnlyKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeCreationDateKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeURLForRemountingKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeUUIDStringKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeNameKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeLocalizedNameKey, str)
        self.assertIsInstance(Foundation.NSURLIsUbiquitousItemKey, str)
        self.assertIsInstance(
            Foundation.NSURLUbiquitousItemHasUnresolvedConflictsKey, str
        )
        self.assertIsInstance(Foundation.NSURLUbiquitousItemIsDownloadedKey, str)
        self.assertIsInstance(Foundation.NSURLUbiquitousItemIsDownloadingKey, str)
        self.assertIsInstance(Foundation.NSURLUbiquitousItemIsUploadedKey, str)
        self.assertIsInstance(Foundation.NSURLUbiquitousItemIsUploadingKey, str)
        self.assertIsInstance(Foundation.NSURLUbiquitousItemPercentDownloadedKey, str)
        self.assertIsInstance(Foundation.NSURLUbiquitousItemPercentUploadedKey, str)

    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertIsInstance(Foundation.NSURLIsExcludedFromBackupKey, str)
        self.assertIsInstance(Foundation.NSURLPathKey, str)

    @min_os_level("10.9")
    def testConstants10_9(self):
        self.assertIsInstance(Foundation.NSURLTagNamesKey, str)
        self.assertIsInstance(Foundation.NSURLUbiquitousItemDownloadingStatusKey, str)
        self.assertIsInstance(Foundation.NSURLUbiquitousItemDownloadingErrorKey, str)
        self.assertIsInstance(Foundation.NSURLUbiquitousItemUploadingErrorKey, str)
        self.assertIsInstance(
            Foundation.NSURLUbiquitousItemDownloadingStatusNotDownloaded, str
        )
        self.assertIsInstance(
            Foundation.NSURLUbiquitousItemDownloadingStatusDownloaded, str
        )
        self.assertIsInstance(
            Foundation.NSURLUbiquitousItemDownloadingStatusCurrent, str
        )

    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertIsInstance(Foundation.NSURLGenerationIdentifierKey, str)
        self.assertIsInstance(Foundation.NSURLDocumentIdentifierKey, str)
        self.assertIsInstance(Foundation.NSURLAddedToDirectoryDateKey, str)
        self.assertIsInstance(Foundation.NSURLQuarantinePropertiesKey, str)
        self.assertIsInstance(Foundation.NSURLThumbnailDictionaryKey, str)
        self.assertIsInstance(Foundation.NSURLThumbnailKey, str)
        self.assertIsInstance(Foundation.NSThumbnail1024x1024SizeKey, str)
        self.assertIsInstance(Foundation.NSURLUbiquitousItemDownloadRequestedKey, str)
        self.assertIsInstance(
            Foundation.NSURLUbiquitousItemContainerDisplayNameKey, str
        )

    @min_os_level("10.11")
    def testConstants10_11(self):
        self.assertIsInstance(Foundation.NSURLIsApplicationKey, str)
        self.assertIsInstance(Foundation.NSURLApplicationIsScriptableKey, str)

    @min_os_level("10.12")
    def testConstants10_12(self):
        self.assertIsInstance(Foundation.NSURLVolumeIsEncryptedKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeIsRootFileSystemKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeSupportsCompressionKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeSupportsFileCloningKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeSupportsSwapRenamingKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeSupportsExclusiveRenamingKey, str)
        self.assertIsInstance(Foundation.NSURLCanonicalPathKey, str)
        self.assertIsInstance(Foundation.NSURLUbiquitousItemIsSharedKey, str)
        self.assertIsInstance(
            Foundation.NSURLUbiquitousSharedItemCurrentUserRoleKey, str
        )
        self.assertIsInstance(
            Foundation.NSURLUbiquitousSharedItemCurrentUserPermissionsKey, str
        )
        self.assertIsInstance(
            Foundation.NSURLUbiquitousSharedItemOwnerNameComponentsKey, str
        )
        self.assertIsInstance(
            Foundation.NSURLUbiquitousSharedItemMostRecentEditorNameComponentsKey, str
        )
        self.assertIsInstance(Foundation.NSURLUbiquitousSharedItemRoleOwner, str)
        self.assertIsInstance(Foundation.NSURLUbiquitousSharedItemRoleParticipant, str)
        self.assertIsInstance(
            Foundation.NSURLUbiquitousSharedItemPermissionsReadOnly, str
        )
        self.assertIsInstance(
            Foundation.NSURLUbiquitousSharedItemPermissionsReadWrite, str
        )

    @min_os_level("10.13")
    def testConstants10_13(self):
        self.assertIsInstance(Foundation.NSURLVolumeSupportsImmutableFilesKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeSupportsAccessPermissionsKey, str)
        self.assertIsInstance(
            Foundation.NSURLVolumeAvailableCapacityForImportantUsageKey, str
        )
        self.assertIsInstance(
            Foundation.NSURLVolumeAvailableCapacityForOpportunisticUsageKey, str
        )

    @min_os_level("11.0")
    def testConstants11_0(self):
        self.assertIsInstance(Foundation.NSURLContentTypeKey, str)

        self.assertIsInstance(Foundation.NSURLFileContentIdentifierKey, str)
        self.assertIsInstance(Foundation.NSURLMayShareFileContentKey, str)
        self.assertIsInstance(Foundation.NSURLMayHaveExtendedAttributesKey, str)
        self.assertIsInstance(Foundation.NSURLIsPurgeableKey, str)
        self.assertIsInstance(Foundation.NSURLIsSparseKey, str)
        self.assertIsInstance(Foundation.NSURLVolumeSupportsFileProtectionKey, str)

        self.assertIsInstance(Foundation.NSURLFileProtectionKey, str)

    @min_os_level("11.3")
    def testConstants11_3(self):
        self.assertIsInstance(Foundation.NSURLUbiquitousItemIsExcludedFromSyncKey, str)

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertArgIsBOOL(
            Foundation.NSURL.URLByAppendingPathComponent_isDirectory_, 1
        )

        self.assertResultIsBOOL(Foundation.NSURL.getResourceValue_forKey_error_)
        self.assertArgIsOut(Foundation.NSURL.getResourceValue_forKey_error_, 0)
        self.assertArgIsOut(Foundation.NSURL.getResourceValue_forKey_error_, 2)
        self.assertArgIsOut(Foundation.NSURL.resourceValuesForKeys_error_, 1)
        self.assertResultIsBOOL(Foundation.NSURL.setResourceValue_forKey_error_)
        self.assertArgIsOut(Foundation.NSURL.setResourceValue_forKey_error_, 2)
        self.assertResultIsBOOL(Foundation.NSURL.setResourceValues_error_)
        self.assertArgIsOut(Foundation.NSURL.setResourceValues_error_, 1)
        self.assertResultIsBOOL(
            Foundation.NSURL.checkResourceIsReachableAndReturnError_
        )
        self.assertArgIsOut(Foundation.NSURL.checkResourceIsReachableAndReturnError_, 0)
        self.assertResultIsBOOL(Foundation.NSURL.isFileReferenceURL)

        self.assertArgIsOut(
            Foundation.NSURL.bookmarkDataWithOptions_includingResourceValuesForKeys_relativeToURL_error_,  # noqa: B950
            3,
        )
        self.assertArgHasType(
            Foundation.NSURL.initByResolvingBookmarkData_options_relativeToURL_bookmarkDataIsStale_error_,  # noqa: B950
            3,
            b"o^" + objc._C_NSBOOL,
        )
        self.assertArgIsOut(
            Foundation.NSURL.initByResolvingBookmarkData_options_relativeToURL_bookmarkDataIsStale_error_,  # noqa: B950
            4,
        )
        self.assertArgHasType(
            Foundation.NSURL.URLByResolvingBookmarkData_options_relativeToURL_bookmarkDataIsStale_error_,  # noqa: B950
            3,
            b"o^" + objc._C_NSBOOL,
        )
        self.assertArgIsOut(
            Foundation.NSURL.URLByResolvingBookmarkData_options_relativeToURL_bookmarkDataIsStale_error_,  # noqa: B950
            4,
        )
        self.assertResultIsBOOL(Foundation.NSURL.writeBookmarkData_toURL_options_error_)
        self.assertArgIsOut(Foundation.NSURL.writeBookmarkData_toURL_options_error_, 3)
        self.assertArgIsOut(Foundation.NSURL.bookmarkDataWithContentsOfURL_error_, 1)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSFileHandle(TestCase):
    def testConstants(self):
        self.assertIsInstance(Foundation.NSFileHandleOperationException, str)
        self.assertIsInstance(Foundation.NSFileHandleReadCompletionNotification, str)
        self.assertIsInstance(
            Foundation.NSFileHandleReadToEndOfFileCompletionNotification, str
        )
        self.assertIsInstance(
            Foundation.NSFileHandleConnectionAcceptedNotification, str
        )
        self.assertIsInstance(Foundation.NSFileHandleDataAvailableNotification, str)
        self.assertIsInstance(Foundation.NSFileHandleNotificationDataItem, str)
        self.assertIsInstance(Foundation.NSFileHandleNotificationFileHandleItem, str)
        self.assertIsInstance(Foundation.NSFileHandleNotificationMonitorModes, str)

    def testMethods(self):
        f = Foundation.NSFileHandle.alloc().initWithFileDescriptor_closeOnDealloc_(
            0, False
        )
        self.assertArgIsBOOL(f.initWithFileDescriptor_closeOnDealloc_, 1)

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertArgIsOut(
            Foundation.NSFileHandle.fileHandleForReadingFromURL_error_, 1
        )
        self.assertArgIsOut(Foundation.NSFileHandle.fileHandleForWritingToURL_error_, 1)
        self.assertArgIsOut(Foundation.NSFileHandle.fileHandleForUpdatingURL_error_, 1)

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertArgIsBlock(Foundation.NSFileHandle.setReadabilityHandler_, 0, b"v@")
        self.assertArgIsBlock(Foundation.NSFileHandle.setWriteabilityHandler_, 0, b"v@")
        self.assertResultIsBlock(Foundation.NSFileHandle.readabilityHandler, b"v@")
        self.assertResultIsBlock(Foundation.NSFileHandle.writeabilityHandler, b"v@")

    @min_os_level("10.15")
    def testMethods10_15(self):
        self.assertArgIsOut(
            Foundation.NSFileHandle.readDataToEndOfFileAndReturnError_, 0
        )
        self.assertArgIsOut(Foundation.NSFileHandle.readDataUpToLength_error_, 1)

        self.assertResultIsBOOL(Foundation.NSFileHandle.writeData_error_)
        self.assertArgIsOut(Foundation.NSFileHandle.writeData_error_, 1)

        self.assertResultIsBOOL(Foundation.NSFileHandle.getOffset_error_)
        self.assertArgIsOut(Foundation.NSFileHandle.getOffset_error_, 0)
        self.assertArgIsOut(Foundation.NSFileHandle.getOffset_error_, 1)

        self.assertResultIsBOOL(Foundation.NSFileHandle.seekToEndReturningOffset_error_)
        self.assertArgIsOut(Foundation.NSFileHandle.seekToEndReturningOffset_error_, 0)
        self.assertArgIsOut(Foundation.NSFileHandle.seekToEndReturningOffset_error_, 1)

        self.assertResultIsBOOL(Foundation.NSFileHandle.seekToOffset_error_)
        self.assertArgIsOut(Foundation.NSFileHandle.seekToOffset_error_, 1)

        self.assertResultIsBOOL(Foundation.NSFileHandle.truncateAtOffset_error_)
        self.assertArgIsOut(Foundation.NSFileHandle.truncateAtOffset_error_, 1)

        self.assertResultIsBOOL(Foundation.NSFileHandle.synchronizeAndReturnError_)
        self.assertArgIsOut(Foundation.NSFileHandle.synchronizeAndReturnError_, 0)

        self.assertResultIsBOOL(Foundation.NSFileHandle.closeAndReturnError_)
        self.assertArgIsOut(Foundation.NSFileHandle.closeAndReturnError_, 0)



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSDistributedNotificationCenter(TestCase):
    def testConstants(self):
        self.assertIsInstance(Foundation.NSLocalNotificationCenterType, str)
        self.assertEqual(Foundation.NSNotificationSuspensionBehaviorDrop, 1)
        self.assertEqual(Foundation.NSNotificationSuspensionBehaviorCoalesce, 2)
        self.assertEqual(Foundation.NSNotificationSuspensionBehaviorHold, 3)
        self.assertEqual(
            Foundation.NSNotificationSuspensionBehaviorDeliverImmediately, 4
        )

        self.assertEqual(Foundation.NSNotificationDeliverImmediately, 1)
        self.assertEqual(Foundation.NSNotificationPostToAllSessions, 2)

        self.assertEqual(Foundation.NSDistributedNotificationDeliverImmediately, 1)
        self.assertEqual(Foundation.NSDistributedNotificationPostToAllSessions, 2)

    def testMethods(self):
        self.assertArgIsSEL(
            Foundation.NSDistributedNotificationCenter.addObserver_selector_name_object_suspensionBehavior_,  # noqa: B950
            1,
            b"v@:@",
        )
        self.assertArgIsSEL(
            Foundation.NSDistributedNotificationCenter.addObserver_selector_name_object_,
            1,
            b"v@:@",
        )

        self.assertArgIsBOOL(
            Foundation.NSDistributedNotificationCenter.postNotificationName_object_userInfo_deliverImmediately_,  # noqa: B950
            3,
        )
        self.assertArgIsBOOL(
            Foundation.NSDistributedNotificationCenter.setSuspended_, 0
        )
        self.assertResultIsBOOL(Foundation.NSDistributedNotificationCenter.suspended)



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestXMLDocument(TestCase):
    def testConstants(self):
        self.assertEqual(Foundation.NSXMLDocumentXMLKind, 0)
        self.assertEqual(Foundation.NSXMLDocumentXHTMLKind, 1)
        self.assertEqual(Foundation.NSXMLDocumentHTMLKind, 2)
        self.assertEqual(Foundation.NSXMLDocumentTextKind, 3)

    def testOutputArgs(self):
        self.assertArgIsOut(
            Foundation.NSXMLDocument.initWithXMLString_options_error_, 2
        )
        self.assertArgIsOut(
            Foundation.NSXMLDocument.initWithContentsOfURL_options_error_, 2
        )
        self.assertArgIsOut(Foundation.NSXMLDocument.initWithData_options_error_, 2)
        self.assertArgIsOut(
            Foundation.NSXMLDocument.objectByApplyingXSLT_arguments_error_, 2
        )
        self.assertArgIsOut(
            Foundation.NSXMLDocument.objectByApplyingXSLTString_arguments_error_, 2
        )
        self.assertArgIsOut(
            Foundation.NSXMLDocument.objectByApplyingXSLTAtURL_arguments_error_, 2
        )
        self.assertArgIsOut(Foundation.NSXMLDocument.validateAndReturnError_, 0)

        self.assertResultIsBOOL(Foundation.NSXMLDocument.isStandalone)
        self.assertArgIsBOOL(Foundation.NSXMLDocument.setStandalone_, 0)
        self.assertResultIsBOOL(Foundation.NSXMLDocument.validateAndReturnError_)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSURLCache(TestCase):
    def testConstants(self):
        self.assertEqual(Foundation.NSURLCacheStorageAllowed, 0)
        self.assertEqual(Foundation.NSURLCacheStorageAllowedInMemoryOnly, 1)
        self.assertEqual(Foundation.NSURLCacheStorageNotAllowed, 2)

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertArgIsBlock(
            Foundation.NSURLCache.getCachedResponseForDataTask_completionHandler_,
            1,
            b"v@",
        )



import Foundation
from PyObjCTools.TestSupport import TestCase


class PythonListAsValue(TestCase):
    def testSettingPythonList(self):
        defaults = Foundation.NSUserDefaults.standardUserDefaults()
        defaults.setObject_forKey_(["a", "b", "c"], "randomKey")

        self.assertEqual(defaults.arrayForKey_("randomKey"), ["a", "b", "c"])

    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSUserDefaults.boolForKey_)
        self.assertArgIsBOOL(Foundation.NSUserDefaults.setBool_forKey_, 0)
        self.assertResultIsBOOL(Foundation.NSUserDefaults.synchronize)
        self.assertResultIsBOOL(Foundation.NSUserDefaults.objectIsForcedForKey_)
        self.assertResultIsBOOL(
            Foundation.NSUserDefaults.objectIsForcedForKey_inDomain_
        )

    def testConstants(self):
        self.assertIsInstance(Foundation.NSGlobalDomain, str)
        self.assertIsInstance(Foundation.NSArgumentDomain, str)
        self.assertIsInstance(Foundation.NSRegistrationDomain, str)

        self.assertIsInstance(Foundation.NSUserDefaultsDidChangeNotification, str)

        self.assertIsInstance(Foundation.NSWeekDayNameArray, str)
        self.assertIsInstance(Foundation.NSShortWeekDayNameArray, str)
        self.assertIsInstance(Foundation.NSMonthNameArray, str)
        self.assertIsInstance(Foundation.NSShortMonthNameArray, str)
        self.assertIsInstance(Foundation.NSTimeFormatString, str)
        self.assertIsInstance(Foundation.NSDateFormatString, str)
        self.assertIsInstance(Foundation.NSTimeDateFormatString, str)
        self.assertIsInstance(Foundation.NSShortTimeDateFormatString, str)
        self.assertIsInstance(Foundation.NSCurrencySymbol, str)
        self.assertIsInstance(Foundation.NSDecimalSeparator, str)
        self.assertIsInstance(Foundation.NSThousandsSeparator, str)
        self.assertIsInstance(Foundation.NSDecimalDigits, str)
        self.assertIsInstance(Foundation.NSAMPMDesignation, str)
        self.assertIsInstance(Foundation.NSHourNameDesignations, str)
        self.assertIsInstance(Foundation.NSYearMonthWeekDesignations, str)
        self.assertIsInstance(Foundation.NSEarlierTimeDesignations, str)
        self.assertIsInstance(Foundation.NSLaterTimeDesignations, str)
        self.assertIsInstance(Foundation.NSThisDayDesignations, str)
        self.assertIsInstance(Foundation.NSNextDayDesignations, str)
        self.assertIsInstance(Foundation.NSNextNextDayDesignations, str)
        self.assertIsInstance(Foundation.NSPriorDayDesignations, str)
        self.assertIsInstance(Foundation.NSDateTimeOrdering, str)
        self.assertIsInstance(Foundation.NSInternationalCurrencyString, str)
        self.assertIsInstance(Foundation.NSShortDateFormatString, str)
        self.assertIsInstance(Foundation.NSPositiveCurrencyFormatString, str)
        self.assertIsInstance(Foundation.NSNegativeCurrencyFormatString, str)



import AppKit
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class PopoverHelper(AppKit.NSObject):
    def popoverShouldClose_(self, a):
        return 1

    def popoverShouldDetach_(self, a):
        return 1


class TestNSPopover(TestCase):
    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(AppKit.NSPopoverAppearanceMinimal, 0)
        self.assertEqual(AppKit.NSPopoverAppearanceHUD, 1)

        self.assertEqual(AppKit.NSPopoverBehaviorApplicationDefined, 0)
        self.assertEqual(AppKit.NSPopoverBehaviorTransient, 1)
        self.assertEqual(AppKit.NSPopoverBehaviorSemitransient, 2)

        self.assertIsInstance(AppKit.NSPopoverCloseReasonKey, str)
        self.assertIsInstance(AppKit.NSPopoverCloseReasonStandard, str)
        self.assertIsInstance(AppKit.NSPopoverCloseReasonDetachToWindow, str)
        self.assertIsInstance(AppKit.NSPopoverWillShowNotification, str)
        self.assertIsInstance(AppKit.NSPopoverDidShowNotification, str)
        self.assertIsInstance(AppKit.NSPopoverWillCloseNotification, str)
        self.assertIsInstance(AppKit.NSPopoverDidCloseNotification, str)

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBOOL(AppKit.NSPopover.animates)
        self.assertArgIsBOOL(AppKit.NSPopover.setAnimates_, 0)

        self.assertResultIsBOOL(AppKit.NSPopover.isShown)
        self.assertArgIsBOOL(AppKit.NSPopover.setShown_, 0)

        self.assertArgHasType(
            AppKit.NSPopover.showRelativeToRect_ofView_preferredEdge_,
            0,
            AppKit.NSRect.__typestr__,
        )

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(AppKit.NSPopover.isDetached)
        # self.assertArgIsBOOL(AppKit.NSPopover.setDetached_, 0)

    @min_os_level("10.7")
    def testProtocols10_7(self):
        self.assertResultIsBOOL(PopoverHelper.popoverShouldClose_)

    @min_os_level("10.10")
    def testProtocols10_10(self):
        objc.protocolNamed("NSPopoverDelegate")
        self.assertResultIsBOOL(PopoverHelper.popoverShouldDetach_)



import CoreFoundation
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestCharacterSet(TestCase):
    def testTypes(self):
        cls = None
        try:
            cls = objc.lookUpClass("__NSCFCharacterSet")
        except objc.error:
            cls = objc.lookUpClass("NSCFCharacterSet")
        if cls is None:
            self.assertIsCFType(CoreFoundation.CFCharacterSetRef)

        else:
            self.assertIs(CoreFoundation.CFCharacterSetRef, cls)

        self.assertIsCFType(CoreFoundation.CFMutableCharacterSetRef)

    def testTypeId(self):
        v = CoreFoundation.CFCharacterSetGetTypeID()
        self.assertIsInstance(v, int)

    def testCreation(self):
        charset = CoreFoundation.CFCharacterSetGetPredefined(
            CoreFoundation.kCFCharacterSetLetter
        )
        self.assertIsInstance(
            charset,
            (CoreFoundation.CFCharacterSetRef, CoreFoundation.CFMutableCharacterSetRef),
        )
        charset = CoreFoundation.CFCharacterSetCreateWithCharactersInRange(
            None, (ord("A"), ord("Z") - ord("A"))
        )
        self.assertIsInstance(
            charset,
            (CoreFoundation.CFCharacterSetRef, CoreFoundation.CFMutableCharacterSetRef),
        )
        charset = CoreFoundation.CFCharacterSetCreateWithCharactersInString(
            None, "abcdefABCDEF0123456789"
        )
        self.assertIsInstance(
            charset,
            (CoreFoundation.CFCharacterSetRef, CoreFoundation.CFMutableCharacterSetRef),
        )
        bytes_value = b"0123" * (8192 // 4)
        charset = CoreFoundation.CFCharacterSetCreateWithBitmapRepresentation(
            None, bytes_value
        )
        self.assertIsInstance(
            charset,
            (CoreFoundation.CFCharacterSetRef, CoreFoundation.CFMutableCharacterSetRef),
        )
        charset = CoreFoundation.CFCharacterSetCreateInvertedSet(None, charset)
        self.assertIsInstance(
            charset,
            (CoreFoundation.CFCharacterSetRef, CoreFoundation.CFMutableCharacterSetRef),
        )
        charset = CoreFoundation.CFCharacterSetCreateMutable(None)
        self.assertIsInstance(
            charset,
            (CoreFoundation.CFCharacterSetRef, CoreFoundation.CFMutableCharacterSetRef),
        )
        charset = CoreFoundation.CFCharacterSetCreateCopy(None, charset)
        self.assertIsInstance(
            charset,
            (CoreFoundation.CFCharacterSetRef, CoreFoundation.CFMutableCharacterSetRef),
        )
        charset = CoreFoundation.CFCharacterSetCreateMutableCopy(None, charset)
        self.assertIsInstance(
            charset,
            (CoreFoundation.CFCharacterSetRef, CoreFoundation.CFMutableCharacterSetRef),
        )

    def testInspection(self):
        letters = CoreFoundation.CFCharacterSetGetPredefined(
            CoreFoundation.kCFCharacterSetLetter
        )
        digits = CoreFoundation.CFCharacterSetGetPredefined(
            CoreFoundation.kCFCharacterSetDecimalDigit
        )
        charset = CoreFoundation.CFCharacterSetCreateWithCharactersInString(
            None, "abcdef"
        )

        self.assertTrue(CoreFoundation.CFCharacterSetIsSupersetOfSet(letters, charset))
        self.assertFalse(CoreFoundation.CFCharacterSetIsSupersetOfSet(digits, charset))

        self.assertTrue(CoreFoundation.CFCharacterSetHasMemberInPlane(digits, 0))
        self.assertFalse(CoreFoundation.CFCharacterSetHasMemberInPlane(digits, 4))

        self.assertTrue(CoreFoundation.CFCharacterSetIsCharacterMember(letters, "A"))
        self.assertFalse(CoreFoundation.CFCharacterSetIsCharacterMember(letters, "9"))

        data = CoreFoundation.CFCharacterSetCreateBitmapRepresentation(None, charset)
        self.assertIsInstance(data, CoreFoundation.CFDataRef)

    def testInspectLongUnicode(self):
        letters = CoreFoundation.CFCharacterSetGetPredefined(
            CoreFoundation.kCFCharacterSetLetter
        )
        digits = CoreFoundation.CFCharacterSetGetPredefined(
            CoreFoundation.kCFCharacterSetDecimalDigit
        )
        self.assertTrue(
            CoreFoundation.CFCharacterSetIsLongCharacterMember(letters, ord("A"))
        )
        self.assertFalse(
            CoreFoundation.CFCharacterSetIsLongCharacterMember(letters, ord("9"))
        )
        self.assertTrue(
            CoreFoundation.CFCharacterSetIsLongCharacterMember(digits, ord("9"))
        )
        self.assertFalse(
            CoreFoundation.CFCharacterSetIsLongCharacterMember(digits, ord("A"))
        )

    def testMutation(self):
        charset = CoreFoundation.CFCharacterSetCreateWithCharactersInString(
            None, "abcdef"
        )
        charset = CoreFoundation.CFCharacterSetCreateMutableCopy(None, charset)

        self.assertFalse(
            CoreFoundation.CFCharacterSetIsCharacterMember(charset, chr(4))
        )
        CoreFoundation.CFCharacterSetAddCharactersInRange(charset, (1, 10))
        self.assertTrue(CoreFoundation.CFCharacterSetIsCharacterMember(charset, chr(4)))

        CoreFoundation.CFCharacterSetRemoveCharactersInRange(charset, (4, 2))
        self.assertFalse(
            CoreFoundation.CFCharacterSetIsCharacterMember(charset, chr(4))
        )

        self.assertFalse(CoreFoundation.CFCharacterSetIsCharacterMember(charset, "5"))
        CoreFoundation.CFCharacterSetAddCharactersInString(charset, "012345")
        self.assertTrue(CoreFoundation.CFCharacterSetIsCharacterMember(charset, "5"))

        self.assertTrue(CoreFoundation.CFCharacterSetIsCharacterMember(charset, "a"))
        CoreFoundation.CFCharacterSetRemoveCharactersInString(charset, "ab")
        self.assertFalse(CoreFoundation.CFCharacterSetIsCharacterMember(charset, "a"))

        self.assertFalse(CoreFoundation.CFCharacterSetIsCharacterMember(charset, "9"))
        CoreFoundation.CFCharacterSetUnion(
            charset,
            CoreFoundation.CFCharacterSetGetPredefined(
                CoreFoundation.kCFCharacterSetDecimalDigit
            ),
        )
        self.assertTrue(CoreFoundation.CFCharacterSetIsCharacterMember(charset, "9"))

        CoreFoundation.CFCharacterSetIntersect(
            charset,
            CoreFoundation.CFCharacterSetGetPredefined(
                CoreFoundation.kCFCharacterSetLetter
            ),
        )
        self.assertFalse(CoreFoundation.CFCharacterSetIsCharacterMember(charset, "9"))

        CoreFoundation.CFCharacterSetInvert(charset)
        self.assertTrue(CoreFoundation.CFCharacterSetIsCharacterMember(charset, "9"))
        self.assertFalse(CoreFoundation.CFCharacterSetIsCharacterMember(charset, "e"))

    def testConstants(self):
        self.assertEqual(CoreFoundation.kCFCharacterSetControl, 1)
        self.assertEqual(CoreFoundation.kCFCharacterSetWhitespace, 2)
        self.assertEqual(CoreFoundation.kCFCharacterSetWhitespaceAndNewline, 3)
        self.assertEqual(CoreFoundation.kCFCharacterSetDecimalDigit, 4)
        self.assertEqual(CoreFoundation.kCFCharacterSetLetter, 5)
        self.assertEqual(CoreFoundation.kCFCharacterSetLowercaseLetter, 6)
        self.assertEqual(CoreFoundation.kCFCharacterSetUppercaseLetter, 7)
        self.assertEqual(CoreFoundation.kCFCharacterSetNonBase, 8)
        self.assertEqual(CoreFoundation.kCFCharacterSetDecomposable, 9)
        self.assertEqual(CoreFoundation.kCFCharacterSetAlphaNumeric, 10)
        self.assertEqual(CoreFoundation.kCFCharacterSetPunctuation, 11)
        self.assertEqual(CoreFoundation.kCFCharacterSetCapitalizedLetter, 13)
        self.assertEqual(CoreFoundation.kCFCharacterSetSymbol, 14)
        self.assertEqual(CoreFoundation.kCFCharacterSetIllegal, 12)

    @min_os_level("10.5")
    def testConstants10_5(self):
        self.assertEqual(CoreFoundation.kCFCharacterSetNewline, 15)



#
#  FilteringArrayController.py
#  TableModelWithSearch
#
#  Created by Bill Bumgarner on Sun Apr 04 2004.
#  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
#

import re

import objc
from Cocoa import NSArrayController
from objc import super

kLiteralSearch = "Literal Search"
kRegularExpressionSearch = "Regular Expression Search"


def regexForSearchString(searchString, searchType):
    if not searchString:
        return None

    searchString = searchString.strip()
    if searchType == kLiteralSearch:
        searchString = re.escape(searchString.strip()) + r"(?i)"
    return re.compile(searchString)


def dictValueFilter(dicts, regex):
    for dct in dicts:
        for value in dct.values():
            if regex.search(value):
                yield dct
                break


class FilteringArrayController(NSArrayController):
    searchString = None
    lastRegex = None
    searchType = kLiteralSearch

    def arrangeObjects_(self, objects):
        supermethod = super().arrangeObjects_
        try:
            regex = regexForSearchString(self.searchString, self.searchType)
        except:  # noqa: E722, B001
            regex = self.lastRegex
        self.lastRegex = regex
        if regex is None:
            return supermethod(objects)
        return supermethod(list(dictValueFilter(objects, regex)))

    @objc.IBAction
    def performSearch_(self, sender):
        self.searchString = sender.stringValue()
        self.rearrangeObjects()

    @objc.IBAction
    def changeSearchType_(self, searchType):
        self.lastRegex = None
        self.searchString = None
        self.searchType = searchType
        self.rearrangeObjects()



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestXMLDTDNode(TestCase):
    def testConstants(self):
        self.assertEqual(Foundation.NSXMLEntityGeneralKind, 1)
        self.assertEqual(Foundation.NSXMLEntityParsedKind, 2)
        self.assertEqual(Foundation.NSXMLEntityUnparsedKind, 3)
        self.assertEqual(Foundation.NSXMLEntityParameterKind, 4)
        self.assertEqual(Foundation.NSXMLEntityPredefined, 5)
        self.assertEqual(Foundation.NSXMLAttributeCDATAKind, 6)
        self.assertEqual(Foundation.NSXMLAttributeIDKind, 7)
        self.assertEqual(Foundation.NSXMLAttributeIDRefKind, 8)
        self.assertEqual(Foundation.NSXMLAttributeIDRefsKind, 9)
        self.assertEqual(Foundation.NSXMLAttributeEntityKind, 10)
        self.assertEqual(Foundation.NSXMLAttributeEntitiesKind, 11)
        self.assertEqual(Foundation.NSXMLAttributeNMTokenKind, 12)
        self.assertEqual(Foundation.NSXMLAttributeNMTokensKind, 13)
        self.assertEqual(Foundation.NSXMLAttributeEnumerationKind, 14)
        self.assertEqual(Foundation.NSXMLAttributeNotationKind, 15)
        self.assertEqual(Foundation.NSXMLElementDeclarationUndefinedKind, 16)
        self.assertEqual(Foundation.NSXMLElementDeclarationEmptyKind, 17)
        self.assertEqual(Foundation.NSXMLElementDeclarationAnyKind, 18)
        self.assertEqual(Foundation.NSXMLElementDeclarationMixedKind, 19)
        self.assertEqual(Foundation.NSXMLElementDeclarationElementKind, 20)

    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSXMLDTDNode.isExternal)



import AppKit
import objc
from PyObjCTools.TestSupport import TestCase


class TestNSUserInterfaceValidationHelper(AppKit.NSObject):
    def action(self):
        return 1

    def tag(self):
        return 1

    def validateUserInterfaceItem_(self, a):
        return 1


class TestNSUserInterfaceValidation(TestCase):
    def testProtocols(self):
        objc.protocolNamed("NSValidatedUserInterfaceItem")
        self.assertResultHasType(
            TestNSUserInterfaceValidationHelper.action, objc._C_SEL
        )
        self.assertResultHasType(
            TestNSUserInterfaceValidationHelper.tag, objc._C_NSInteger
        )

        objc.protocolNamed("NSUserInterfaceValidations")
        self.assertResultIsBOOL(
            TestNSUserInterfaceValidationHelper.validateUserInterfaceItem_
        )



import AppKit
import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSGeometry(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSMinXEdge, 0)
        self.assertEqual(AppKit.NSMinYEdge, 1)
        self.assertEqual(AppKit.NSMaxXEdge, 2)
        self.assertEqual(AppKit.NSMaxYEdge, 3)

        self.assertEqual(AppKit.NSRectEdgeMinX, 0)
        self.assertEqual(AppKit.NSRectEdgeMinY, 1)
        self.assertEqual(AppKit.NSRectEdgeMaxX, 2)
        self.assertEqual(AppKit.NSRectEdgeMaxY, 3)

        self.assertEqual(AppKit.NSZeroPoint, AppKit.NSPoint())
        self.assertEqual(AppKit.NSZeroSize, AppKit.NSSize())
        self.assertEqual(AppKit.NSZeroRect, AppKit.NSRect())

    def testInlines(self):
        self.assertEqual(AppKit.NSMakePoint(1, 2), AppKit.NSPoint(1, 2))
        self.assertEqual(AppKit.NSMakeSize(4, 5), AppKit.NSSize(4, 5))
        self.assertEqual(
            AppKit.NSMakeRect(1, 2, 3, 4),
            AppKit.NSRect(AppKit.NSPoint(1, 2), AppKit.NSSize(3, 4)),
        )

        rect = AppKit.NSRect(AppKit.NSPoint(1, 2), AppKit.NSSize(4, 6))
        self.assertEqual(AppKit.NSMaxX(rect), 5)
        self.assertEqual(AppKit.NSMaxY(rect), 8)
        self.assertEqual(AppKit.NSMidX(rect), 3)
        self.assertEqual(AppKit.NSMidY(rect), 5)
        self.assertEqual(AppKit.NSMinX(rect), 1)
        self.assertEqual(AppKit.NSMinY(rect), 2)
        self.assertEqual(AppKit.NSWidth(rect), 4)
        self.assertEqual(AppKit.NSHeight(rect), 6)

        # Cannot test these here, need to be tested in the Quartz unittests
        self.assertHasAttr(Foundation, "NSRectFromCGRect")
        self.assertHasAttr(Foundation, "NSRectToCGRect")
        self.assertHasAttr(Foundation, "NSPointFromCGPoint")
        self.assertHasAttr(Foundation, "NSPointToCGPoint")
        self.assertHasAttr(Foundation, "NSSizeFromCGSize")
        self.assertHasAttr(Foundation, "NSSizeToCGSize")

    @min_os_level("10.8")
    def testInlines10_8(self):
        v = AppKit.NSEdgeInsetsMake(1, 2, 3, 4)
        self.assertIsInstance(v, AppKit.NSEdgeInsets)
        self.assertEqual(v.top, 1.0)
        self.assertEqual(v.left, 2.0)
        self.assertEqual(v.bottom, 3.0)
        self.assertEqual(v.right, 4.0)

    def testFunctions(self):
        p1 = AppKit.NSPoint(1, 2)
        p2 = AppKit.NSPoint(3, 4)

        s1 = AppKit.NSSize(4, 5)
        s2 = AppKit.NSSize(7, 8)

        r0 = AppKit.NSRect(AppKit.NSPoint(9, 10), AppKit.NSSize(0, 0))
        r1 = AppKit.NSRect(AppKit.NSPoint(0, 1), AppKit.NSSize(4, 5))
        r2 = AppKit.NSRect(AppKit.NSPoint(4.5, 5.5), AppKit.NSSize(7.5, 8.5))

        self.assertIs(AppKit.NSEqualPoints(p1, p1), True)
        self.assertIs(AppKit.NSEqualPoints(p1, p2), False)
        self.assertIs(AppKit.NSEqualSizes(s1, s1), True)
        self.assertIs(AppKit.NSEqualSizes(s1, s2), False)
        self.assertIs(AppKit.NSEqualRects(r1, r1), True)
        self.assertIs(AppKit.NSEqualRects(r1, r2), False)
        self.assertIs(AppKit.NSIsEmptyRect(r0), True)
        self.assertIs(AppKit.NSIsEmptyRect(r1), False)
        ra = AppKit.NSInsetRect(r1, 2, 3)
        self.assertEqual(ra, AppKit.NSRect(AppKit.NSPoint(2, 4), AppKit.NSSize(0, -1)))

        ra = AppKit.NSIntegralRect(r2)
        self.assertEqual(ra, AppKit.NSRect(AppKit.NSPoint(4, 5), AppKit.NSSize(8, 9)))

        ra = AppKit.NSUnionRect(r1, r2)
        self.assertEqual(ra, AppKit.NSRect(AppKit.NSPoint(0, 1), AppKit.NSSize(12, 13)))

        ra = AppKit.NSIntersectionRect(r1, r2)
        self.assertEqual(ra, AppKit.NSRect(AppKit.NSPoint(0, 0), AppKit.NSSize(0, 0)))

        ra = AppKit.NSOffsetRect(r1, 5, 6)
        self.assertEqual(ra, AppKit.NSRect(AppKit.NSPoint(5, 7), AppKit.NSSize(4, 5)))

        slice_value, rem = AppKit.NSDivideRect(r2, None, None, 1.5, AppKit.NSMaxYEdge)
        self.assertEqual(
            slice_value,
            AppKit.NSRect(AppKit.NSPoint(4.5, 12.5), AppKit.NSSize(7.5, 1.5)),
        )
        self.assertEqual(
            rem, AppKit.NSRect(AppKit.NSPoint(4.5, 5.5), AppKit.NSSize(7.5, 7.0))
        )

        self.assertIs(AppKit.NSPointInRect(p1, r1), True)
        self.assertIs(AppKit.NSMouseInRect(p1, r1, False), True)
        self.assertIs(AppKit.NSMouseInRect(p2, r2, True), False)
        self.assertIs(AppKit.NSContainsRect(r1, r2), False)
        self.assertIs(AppKit.NSIntersectsRect(r1, r2), False)
        self.assertEqual(AppKit.NSStringFromPoint(p1), "{1, 2}")
        self.assertEqual(AppKit.NSStringFromSize(s1), "{4, 5}")
        self.assertEqual(AppKit.NSStringFromRect(r1), "{{0, 1}, {4, 5}}")
        v = AppKit.NSPointFromString("{1, 2}")
        self.assertEqual(v, p1)
        v = AppKit.NSSizeFromString("{4,5}")
        self.assertEqual(v, s1)
        v = AppKit.NSRectFromString("{   {0,1}  , {  4, 5}}")
        self.assertEqual(v, r1)

    def testValueMethods(self):
        v = AppKit.NSValue.valueWithPoint_(AppKit.NSPoint(2, 3))
        w = v.pointValue()
        self.assertIsInstance(w, AppKit.NSPoint)
        self.assertEqual(w, (2, 3))

        w = v.sizeValue()
        self.assertIsInstance(w, AppKit.NSSize)
        self.assertEqual(w, (2, 3))

        v = AppKit.NSValue.valueWithSize_(AppKit.NSSize(9, 8))
        w = v.sizeValue()
        self.assertIsInstance(w, AppKit.NSSize)
        self.assertEqual(w, (9, 8))

        v = AppKit.NSValue.valueWithRect_(
            AppKit.NSRect(AppKit.NSPoint(9, 10), AppKit.NSSize(11, 12))
        )
        w = v.rectValue()
        self.assertIsInstance(w, AppKit.NSRect)
        self.assertEqual(w, ((9, 10), (11, 12)))

        self.assertArgHasType(
            AppKit.NSValue.valueWithPoint_, 0, AppKit.NSPoint.__typestr__
        )
        self.assertResultHasType(AppKit.NSValue.pointValue, AppKit.NSPoint.__typestr__)
        self.assertArgHasType(
            AppKit.NSValue.valueWithSize_, 0, AppKit.NSSize.__typestr__
        )
        self.assertResultHasType(AppKit.NSValue.sizeValue, AppKit.NSSize.__typestr__)
        self.assertArgHasType(
            AppKit.NSValue.valueWithRect_, 0, AppKit.NSRect.__typestr__
        )
        self.assertResultHasType(AppKit.NSValue.rectValue, AppKit.NSRect.__typestr__)

    def testCoderMethods(self):
        self.assertArgHasType(
            AppKit.NSCoder.encodePoint_, 0, AppKit.NSPoint.__typestr__
        )
        self.assertResultHasType(AppKit.NSCoder.decodePoint, AppKit.NSPoint.__typestr__)
        self.assertArgHasType(
            AppKit.NSCoder.encodePoint_forKey_, 0, AppKit.NSPoint.__typestr__
        )
        self.assertResultHasType(
            AppKit.NSCoder.decodePointForKey_, AppKit.NSPoint.__typestr__
        )

        self.assertArgHasType(AppKit.NSCoder.encodeSize_, 0, AppKit.NSSize.__typestr__)
        self.assertResultHasType(AppKit.NSCoder.decodeSize, AppKit.NSSize.__typestr__)
        self.assertArgHasType(
            AppKit.NSCoder.encodeSize_forKey_, 0, AppKit.NSSize.__typestr__
        )
        self.assertResultHasType(
            AppKit.NSCoder.decodeSizeForKey_, AppKit.NSSize.__typestr__
        )

        self.assertArgHasType(AppKit.NSCoder.encodeRect_, 0, AppKit.NSRect.__typestr__)
        self.assertResultHasType(AppKit.NSCoder.decodeRect, AppKit.NSRect.__typestr__)
        self.assertArgHasType(
            AppKit.NSCoder.encodeRect_forKey_, 0, AppKit.NSRect.__typestr__
        )
        self.assertResultHasType(
            AppKit.NSCoder.decodeRectForKey_, AppKit.NSRect.__typestr__
        )

    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertIsInstance(AppKit.NSEdgeInsetsZero, AppKit.NSEdgeInsets)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(AppKit.NSAlignMinXInward, 1 << 0)
        self.assertEqual(AppKit.NSAlignMinYInward, 1 << 1)
        self.assertEqual(AppKit.NSAlignMaxXInward, 1 << 2)
        self.assertEqual(AppKit.NSAlignMaxYInward, 1 << 3)
        self.assertEqual(AppKit.NSAlignWidthInward, 1 << 4)
        self.assertEqual(AppKit.NSAlignHeightInward, 1 << 5)
        self.assertEqual(AppKit.NSAlignMinXOutward, 1 << 8)
        self.assertEqual(AppKit.NSAlignMinYOutward, 1 << 9)
        self.assertEqual(AppKit.NSAlignMaxXOutward, 1 << 10)
        self.assertEqual(AppKit.NSAlignMaxYOutward, 1 << 11)
        self.assertEqual(AppKit.NSAlignWidthOutward, 1 << 12)
        self.assertEqual(AppKit.NSAlignHeightOutward, 1 << 13)
        self.assertEqual(AppKit.NSAlignMinXNearest, 1 << 16)
        self.assertEqual(AppKit.NSAlignMinYNearest, 1 << 17)
        self.assertEqual(AppKit.NSAlignMaxXNearest, 1 << 18)
        self.assertEqual(AppKit.NSAlignMaxYNearest, 1 << 19)
        self.assertEqual(AppKit.NSAlignWidthNearest, 1 << 20)
        self.assertEqual(AppKit.NSAlignHeightNearest, 1 << 21)
        self.assertEqual(AppKit.NSAlignRectFlipped, 1 << 63)
        self.assertEqual(
            AppKit.NSAlignAllEdgesInward,
            AppKit.NSAlignMinXInward
            | AppKit.NSAlignMaxXInward
            | AppKit.NSAlignMinYInward
            | AppKit.NSAlignMaxYInward,
        )
        self.assertEqual(
            AppKit.NSAlignAllEdgesOutward,
            AppKit.NSAlignMinXOutward
            | AppKit.NSAlignMaxXOutward
            | AppKit.NSAlignMinYOutward
            | AppKit.NSAlignMaxYOutward,
        )
        self.assertEqual(
            AppKit.NSAlignAllEdgesNearest,
            AppKit.NSAlignMinXNearest
            | AppKit.NSAlignMaxXNearest
            | AppKit.NSAlignMinYNearest
            | AppKit.NSAlignMaxYNearest,
        )

    @min_os_level("10.7")
    def testFunctions10_7(self):
        r2 = AppKit.NSRect(AppKit.NSPoint(4.5, 5.5), AppKit.NSSize(7.5, 8.5))
        r = AppKit.NSIntegralRectWithOptions(r2, AppKit.NSAlignAllEdgesNearest)
        self.assertIsInstance(r, AppKit.NSRect)

    @min_os_level("10.10")
    def testFunctions10_10(self):
        self.assertResultIsBOOL(AppKit.NSEdgeInsetsEqual)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSTableColumn(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSTableColumnNoResizing, 0)
        self.assertEqual(AppKit.NSTableColumnAutoresizingMask, (1 << 0))
        self.assertEqual(AppKit.NSTableColumnUserResizingMask, (1 << 1))

    def testMethods(self):
        self.assertArgIsBOOL(AppKit.NSTableColumn.setEditable_, 0)
        self.assertResultIsBOOL(AppKit.NSTableColumn.isEditable)
        self.assertArgIsBOOL(AppKit.NSTableColumn.setResizable_, 0)
        self.assertResultIsBOOL(AppKit.NSTableColumn.isResizable)

    @min_os_level("10.5")
    def testMethods10_5(self):
        self.assertArgIsBOOL(AppKit.NSTableColumn.setHidden_, 0)
        self.assertResultIsBOOL(AppKit.NSTableColumn.isHidden)



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSClassDescription(TestCase):
    def testConstants(self):
        self.assertIsInstance(
            Foundation.NSClassDescriptionNeededForClassNotification, str
        )



#
#  GraphicsArrayController.py
#  GraphicsBindings
#
#  Converted by u.fiedler on feb 2005
#  with great help from Bob Ippolito - Thank you Bob!
#
#  The original version was written in Objective-C by Malcolm Crawford
#  http://homepage.mac.com/mmalc/CocoaExamples/controllers.html

from math import fabs
from random import random

import objc
from Cocoa import NSArrayController, NSCalibratedRGBColorSpace, NSColor
from objc import super


class GraphicsArrayController(NSArrayController):
    """Allow filtering by color, just for the fun of it"""

    filterColor = objc.IBOutlet()
    newCircle = objc.IBOutlet()
    shouldFilter = objc.ivar.BOOL()
    graphicsView = objc.IBOutlet()

    def arrangeObjects_(self, objects):
        """Filtering is not yet connected in IB!"""
        if self.shouldFilter:
            self.shouldFilter = False

        if not self.shouldFilter:
            return super().arrangeObjects_(objects)

        if self.filterColor is None:
            self.filterColor = NSColor.blackColor().colorUsingColorSpaceName_(
                NSCalibratedRGBColorSpace
            )

        filterHue = self.filterColor.hueComponent()
        filteredObjects = []
        for item in objects:
            hue = item.color.hueComponent()
            if (
                (fabs(hue - filterHue) < 0.05)
                or (fabs(hue - filterHue) > 0.95)
                or (item is self.newCircle)
            ):
                filteredObjects.append(item)
                self.newCircle = None
        return super().arrangeObjects_(filteredObjects)

    def newObject(self):
        """Randomize attributes of new circles so we get a pretty display"""
        self.newCircle = super().newObject()
        radius = 5.0 + 15.0 * random()
        self.newCircle.radius = radius

        height = self.graphicsView.bounds().size.height
        width = self.graphicsView.bounds().size.width

        xOffset = 10.0 + (height - 20.0) * random()
        yOffset = 10.0 + (width - 20.0) * random()

        self.newCircle.xLoc = xOffset
        self.newCircle.yLoc = height - yOffset

        color = NSColor.colorWithCalibratedHue_saturation_brightness_alpha_(
            random(), (0.5 + random() / 2.0), (0.333 + random() / 3.0), 1.0
        )

        self.newCircle.color = color
        return self.newCircle



import AppKit
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSRulerViewHelper(AppKit.NSView):
    def rulerView_shouldMoveMarker_(self, rl, mk):
        return 1

    def rulerView_willMoveMarker_toLocation_(self, rl, mk, x):
        return 1

    def rulerView_didMoveMarker_(self, rv, mk):
        pass

    def rulerView_shouldRemoveMarker_(self, rv, mk):
        return 1

    def rulerView_didRemoveMarker_(self, rv, mk):
        pass

    def rulerView_shouldAddMarker_(self, rv, mk):
        return 1

    def rulerView_willAddMarker_atLocation_(self, rv, mk, x):
        return 1

    def rulerView_didAddMarker_(self, rv, mk):
        pass

    def rulerView_handleMouseDown_(self, rv, ev):
        pass

    def rulerView_willSetClientView_(self, rv, vw):
        pass

    def rulerView_locationForPoint_(self, rv, vw):
        return 1

    def rulerView_pointForLocation_(self, rv, vw):
        return 1


class TestNSRulerView(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSHorizontalRuler, 0)
        self.assertEqual(AppKit.NSVerticalRuler, 1)

    @min_os_level("10.13")
    def testConstants10_13(self):
        self.assertIsInstance(AppKit.NSRulerViewUnitInches, str)
        self.assertIsInstance(AppKit.NSRulerViewUnitCentimeters, str)
        self.assertIsInstance(AppKit.NSRulerViewUnitPoints, str)
        self.assertIsInstance(AppKit.NSRulerViewUnitPicas, str)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSRulerView.trackMarker_withMouseEvent_)
        self.assertResultIsBOOL(AppKit.NSRulerView.isFlipped)

    def testProtocols(self):
        self.assertResultIsBOOL(TestNSRulerViewHelper.rulerView_shouldMoveMarker_)

        self.assertResultHasType(
            TestNSRulerViewHelper.rulerView_willMoveMarker_toLocation_, objc._C_CGFloat
        )
        self.assertArgHasType(
            TestNSRulerViewHelper.rulerView_willMoveMarker_toLocation_,
            2,
            objc._C_CGFloat,
        )

        self.assertResultIsBOOL(TestNSRulerViewHelper.rulerView_shouldRemoveMarker_)
        self.assertResultIsBOOL(TestNSRulerViewHelper.rulerView_shouldAddMarker_)
        self.assertResultHasType(
            TestNSRulerViewHelper.rulerView_willAddMarker_atLocation_, objc._C_CGFloat
        )
        self.assertArgHasType(
            TestNSRulerViewHelper.rulerView_willAddMarker_atLocation_,
            2,
            objc._C_CGFloat,
        )

        self.assertResultHasType(
            TestNSRulerViewHelper.rulerView_locationForPoint_, objc._C_CGFloat
        )
        self.assertArgHasType(
            TestNSRulerViewHelper.rulerView_locationForPoint_,
            1,
            AppKit.NSPoint.__typestr__,
        )

        self.assertResultHasType(
            TestNSRulerViewHelper.rulerView_pointForLocation_,
            AppKit.NSPoint.__typestr__,
        )
        self.assertArgHasType(
            TestNSRulerViewHelper.rulerView_pointForLocation_, 1, objc._C_CGFloat
        )



import operator
import os
import re
import sys

import objc
import Foundation
from PyObjCTools.TestSupport import TestCase, sdkForPython

# flake8: noqa: B950
PLIST = b"""\
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
<plist version="1.0">
<dict>
\t<key>bool</key>
\t<true/>
\t<key>plain</key>
\t<integer>1</integer>
</dict>
</plist>
""".decode(
    "latin1"
)


def stripDocType(val):
    """
    Strip non-significant information. This is needed because the proplists
    on macOS 10.1 are slightly different from the ones on macOS 10.2 (
    different DOCTYPE and version).
    """
    r = re.sub("<!DOCTYPE [^>]*>", "<!DOCTYPE>", val)
    return r.replace('version="0.9"', 'version="1.0"')


class TestNSNumber(TestCase):
    def testSimple(self):
        self.assertEqual(Foundation.NSNumber.numberWithFloat_(1.0), 1, 0)
        self.assertEqual(Foundation.NSNumber.numberWithInt_(1), 1)
        self.assertEqual(Foundation.NSNumber.numberWithFloat_(-0.5), -0.5)
        self.assertEqual(Foundation.NSNumber.numberWithInt_(-4), -4)
        self.assertEqual(Foundation.NSNumber.numberWithInt_(0), 0)
        self.assertEqual(Foundation.NSNumber.numberWithFloat_(0.0), 0.0)

    def testReadOnly(self):
        n = Foundation.NSNumber.numberWithFloat_(1.2)
        self.assertRaises(AttributeError, setattr, n, "foo", 2)

        n = Foundation.NSNumber.numberWithInt_(1)
        self.assertRaises(AttributeError, setattr, n, "foo", 2)

        n = Foundation.NSNumber.numberWithLongLong_(2 ** 32 + 2)
        self.assertRaises(AttributeError, setattr, n, "foo", 2)

    def testUseAsBasicType(self):
        lstValue = list(range(0, 20, 2))
        for idx, v in enumerate(lstValue):
            self.assertEqual(v, lstValue[Foundation.NSNumber.numberWithInt_(idx)])
            self.assertEqual(v, lstValue[Foundation.NSNumber.numberWithLong_(idx)])
            self.assertEqual(v, lstValue[Foundation.NSNumber.numberWithLongLong_(idx)])

        self.assertRaises(
            TypeError,
            operator.getitem,
            lstValue,
            Foundation.NSNumber.numberWithFloat_(2.0),
        )

    def testUnsignedIssues(self):
        # Foundation.NSNumber stores unsigned numbers as signed numbers
        # This is a bug in Cocoa... (RADAR #4007594), fixed in 10.5
        if sdkForPython() is not None and sdkForPython() < (10, 5):
            self.assertEqual(
                Foundation.NSNumber.numberWithUnsignedInt_(2 ** 31), -(2 ** 31)
            )
        else:
            self.assertEqual(
                Foundation.NSNumber.numberWithUnsignedInt_(2 ** 31), (2 ** 31)
            )

    def testMethods(self):
        v = Foundation.NSNumber.numberWithUnsignedInt_(2 ** 31)

        self.assertEqual(v.unsignedIntValue(), 2 ** 31)
        self.assertEqual(v.intValue(), -(2 ** 31))

        v = Foundation.NSNumber.numberWithInt_(10)
        self.assertEqual(v.doubleValue(), float(10))

    def testMath(self):
        Xs = list(range(10, 40, 3))
        Ys = list(range(-12, 44, 5))

        self.assertTrue(0 not in Ys)
        self.assertTrue(0 not in Xs)

        for x in Xs:
            for y in Ys:
                Nx = Foundation.NSNumber.numberWithInt_(x)
                Ny = Foundation.NSNumber.numberWithInt_(y)

                self.assertEqual(x + y, Nx + Ny)
                self.assertEqual(x - y, Nx - Ny)
                self.assertEqual(x * y, Nx * Ny)
                self.assertEqual(x / y, Nx / Ny)
                self.assertEqual(x % y, Nx % Ny)
                self.assertEqual(x ** y, Nx ** Ny)

                Nx = Foundation.NSNumber.numberWithFloat_(x + 0.5)
                Ny = Foundation.NSNumber.numberWithFloat_(y + 0.5)

                self.assertEqual((x + 0.5) + (y + 0.5), Nx + Ny)
                self.assertEqual((x + 0.5) - (y + 0.5), Nx - Ny)
                self.assertEqual((x + 0.5) * (y + 0.5), Nx * Ny)
                self.assertEqual((x + 0.5) / (y + 0.5), Nx / Ny)
                self.assertEqual((x + 0.5) % (y + 0.5), Nx % Ny)
                self.assertEqual((x + 0.5) ** (y + 0.5), Nx ** Ny)

                Nx = Foundation.NSNumber.numberWithLongLong_(x)
                Ny = Foundation.NSNumber.numberWithLongLong_(y)

                self.assertEqual((x) + (y), Nx + Ny)
                self.assertEqual((x) - (y), Nx - Ny)
                self.assertEqual((x) * (y), Nx * Ny)
                self.assertEqual((x) / (y), Nx / Ny)
                self.assertEqual((x) % (y), Nx % Ny)
                self.assertEqual((x) ** (y), Nx ** Ny)

    def testTyping(self):
        # Thanks to some tricks and a cooperating Python runtime,
        # Foundation.NSNumber "instances" seem to be subclasses of both Foundation.NSNumber and
        # the corresponding Python number type.
        #

        n = Foundation.NSNumber.numberWithInt_(10)
        self.assertIsInstance(n, int)
        self.assertIsInstance(n, Foundation.NSNumber)

        n = Foundation.NSNumber.numberWithUnsignedInt_(10)
        self.assertIsInstance(n, int)
        self.assertIsInstance(n, Foundation.NSNumber)

        n = Foundation.NSNumber.numberWithLong_(10)
        self.assertIsInstance(n, int)
        self.assertIsInstance(n, Foundation.NSNumber)

        n = Foundation.NSNumber.numberWithUnsignedLong_(10)
        self.assertIsInstance(n, int)
        self.assertIsInstance(n, Foundation.NSNumber)

        n = Foundation.NSNumber.numberWithLongLong_(2 ** 32 * 1024)
        self.assertEqual(n, 2 ** 32 * 1024)
        self.assertIsInstance(n, int)
        self.assertIsInstance(n, Foundation.NSNumber)

        n = Foundation.NSNumber.numberWithUnsignedLongLong_(2 ** 32 + 100)
        self.assertEqual(n, 2 ** 32 + 100)
        self.assertIsInstance(n, int)
        self.assertIsInstance(n, Foundation.NSNumber)

        n = Foundation.NSNumber.numberWithFloat_(10)
        self.assertIsInstance(n, float)
        self.assertIsInstance(n, Foundation.NSNumber)

        n = Foundation.NSNumber.numberWithDouble_(10)
        self.assertIsInstance(n, float)
        self.assertIsInstance(n, Foundation.NSNumber)


if objc.platform == "MACOSX":

    class TestPropList(TestCase):
        # Test if numbers are stored properly in property-list. The most
        # important part of the testcase are boolean values.
        #
        # NOTE: GNUstep uses the old NeXT property lists, and these tests
        # will fail.

        def testPropertyList1(self):
            d = Foundation.NSMutableDictionary.dictionary()

            # Python 2.3 only...
            d["plain"] = 1
            d["bool"] = objc.YES

            self.assertEqual(d.writeToFile_atomically_("/tmp/pyobjctest.plist", 0), 1)

            fd = open("/tmp/pyobjctest.plist", "rb")
            data = fd.read().decode("utf8")
            fd.close()

            self.assertEqual(stripDocType(data), stripDocType(PLIST))

        def testPropertyList2(self):
            d = Foundation.NSMutableDictionary.dictionary()

            d["plain"] = Foundation.NSNumber.numberWithLong_(1)
            d["bool"] = Foundation.NSNumber.numberWithBool_(1)

            self.assertEqual(d.writeToFile_atomically_("/tmp/pyobjctest.plist", 0), 1)

            fd = open("/tmp/pyobjctest.plist", "rb")
            data = fd.read().decode("utf8")
            fd.close()

            self.assertEqual(stripDocType(data), stripDocType(PLIST))


class TestDecimalNumber(TestCase):
    def testProxy(self):
        one = Foundation.NSDecimalNumber.decimalNumberWithString_("1.00")
        self.assertIsInstance(one, Foundation.NSDecimalNumber)

        two = Foundation.NSDecimalNumber.decimalNumberWithString_("2.00")
        self.assertIsInstance(two, Foundation.NSDecimalNumber)

        three = Foundation.NSDecimalNumber.decimalNumberWithString_("3.00")
        self.assertIsInstance(three, Foundation.NSDecimalNumber)

        six = Foundation.NSDecimalNumber.decimalNumberWithString_("6.00")
        self.assertIsInstance(six, Foundation.NSDecimalNumber)

        one_half = Foundation.NSDecimalNumber.decimalNumberWithString_("0.50")
        self.assertIsInstance(one_half, Foundation.NSDecimalNumber)

        self.assertEqual(one + two, three)
        self.assertEqual(three - one, two)
        self.assertEqual(three * two, six)
        self.assertEqual(one / two, one_half)
        self.assertEqual(three // two, one)

        if sys.version_info[0] > 2:
            self.assertEqual(round(three / two), two)
            self.assertEqual(round(one / two, 1), one_half)



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSImageCell(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSImageAlignCenter, 0)
        self.assertEqual(AppKit.NSImageAlignTop, 1)
        self.assertEqual(AppKit.NSImageAlignTopLeft, 2)
        self.assertEqual(AppKit.NSImageAlignTopRight, 3)
        self.assertEqual(AppKit.NSImageAlignLeft, 4)
        self.assertEqual(AppKit.NSImageAlignBottom, 5)
        self.assertEqual(AppKit.NSImageAlignBottomLeft, 6)
        self.assertEqual(AppKit.NSImageAlignBottomRight, 7)
        self.assertEqual(AppKit.NSImageAlignRight, 8)

        self.assertEqual(AppKit.NSImageFrameNone, 0)
        self.assertEqual(AppKit.NSImageFramePhoto, 1)
        self.assertEqual(AppKit.NSImageFrameGrayBezel, 2)
        self.assertEqual(AppKit.NSImageFrameGroove, 3)
        self.assertEqual(AppKit.NSImageFrameButton, 4)



import Foundation
import CoreFoundation
from PyObjCTools.TestSupport import TestCase


class TestTollFreeBridging(TestCase):
    def testImplicitFromCF(self):

        c = CoreFoundation.CFArrayCreateMutable(None, 0, None)
        self.assertIsInstance(c, CoreFoundation.CFMutableArrayRef)

        nsa = Foundation.NSMutableArray.array()
        nsa.addObject_(c)

        o = nsa[0]
        self.assertIsInstance(o, Foundation.NSMutableArray)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSTreeNode(TestCase):
    @min_os_level("10.5")
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSTreeNode.isLeaf)
        self.assertArgIsBOOL(AppKit.NSTreeNode.sortWithSortDescriptors_recursively_, 1)



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSTextFieldCell(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSTextFieldSquareBezel, 0)
        self.assertEqual(AppKit.NSTextFieldRoundedBezel, 1)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSTextFieldCell.drawsBackground)
        self.assertArgIsBOOL(AppKit.NSTextFieldCell.setDrawsBackground_, 0)
        self.assertArgIsBOOL(
            AppKit.NSTextFieldCell.setWantsNotificationForMarkedText_, 0
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSButton(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSButton.isBordered)
        self.assertArgIsBOOL(AppKit.NSButton.setBordered_, 0)
        self.assertResultIsBOOL(AppKit.NSButton.isTransparent)
        self.assertArgIsBOOL(AppKit.NSButton.setTransparent_, 0)
        self.assertArgIsOut(AppKit.NSButton.getPeriodicDelay_interval_, 0)
        self.assertArgIsOut(AppKit.NSButton.getPeriodicDelay_interval_, 1)
        self.assertResultIsBOOL(AppKit.NSButton.performKeyEquivalent_)
        self.assertResultIsBOOL(AppKit.NSButton.allowsMixedState)
        self.assertArgIsBOOL(AppKit.NSButton.setAllowsMixedState_, 0)
        self.assertArgIsBOOL(AppKit.NSButton.setShowsBorderOnlyWhileMouseInside_, 0)
        self.assertResultIsBOOL(AppKit.NSButton.showsBorderOnlyWhileMouseInside)
        self.assertArgIsBOOL(AppKit.NSButton.highlight_, 0)

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(AppKit.NSButton.isSpringLoaded)
        self.assertArgIsBOOL(AppKit.NSButton.setSpringLoaded_, 0)

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertResultIsBOOL(AppKit.NSButton.imageHugsTitle)
        self.assertArgIsBOOL(AppKit.NSButton.setImageHugsTitle_, 0)

        self.assertArgIsSEL(
            AppKit.NSButton.buttonWithTitle_image_target_action_, 3, b"v@:@"
        )
        self.assertArgIsSEL(AppKit.NSButton.buttonWithTitle_target_action_, 2, b"v@:@")
        self.assertArgIsSEL(AppKit.NSButton.buttonWithImage_target_action_, 2, b"v@:@")
        self.assertArgIsSEL(
            AppKit.NSButton.checkboxWithTitle_target_action_, 2, b"v@:@"
        )
        self.assertArgIsSEL(
            AppKit.NSButton.radioButtonWithTitle_target_action_, 2, b"v@:@"
        )



import AppKit
from PyObjCTools.TestSupport import TestCase
import objc


class TestNSController(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSController.commitEditing)
        self.assertResultIsBOOL(AppKit.NSController.isEditing)
        self.assertArgIsSEL(
            AppKit.NSController.commitEditingWithDelegate_didCommitSelector_contextInfo_,
            1,
            b"v@:@" + objc._C_NSBOOL + b"^v",
        )
        self.assertArgHasType(
            AppKit.NSController.commitEditingWithDelegate_didCommitSelector_contextInfo_,
            2,
            b"^v",
        )



import Foundation
import objc
from PyObjCTools.TestSupport import TestCase


class TestNSScriptKeyValueCodingHelper(Foundation.NSObject):
    def insertValue_atIndex_inPropertyWithKey_(self, a, b, c):
        pass

    def removeValueAtIndex_fromPropertyWithKey_(self, a, b):
        pass

    def replaceValueAtIndex_inPropertyWithKey_withValue_(self, a, b, c):
        pass


class TestNSScriptKeyValueCoding(TestCase):
    def testMethods(self):
        self.assertArgHasType(
            TestNSScriptKeyValueCodingHelper.insertValue_atIndex_inPropertyWithKey_,
            1,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSScriptKeyValueCodingHelper.removeValueAtIndex_fromPropertyWithKey_,
            0,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSScriptKeyValueCodingHelper.replaceValueAtIndex_inPropertyWithKey_withValue_,
            0,
            objc._C_NSUInteger,
        )



import objc
import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level


class TestNSUserInterfaceCompression(TestCase):
    @min_os_level("10.13")
    def testMethods(self):
        self.assertResultIsBOOL(
            AppKit.NSUserInterfaceCompressionOptions.containsOptions_
        )
        self.assertResultIsBOOL(
            AppKit.NSUserInterfaceCompressionOptions.intersectsOptions_
        )
        self.assertResultIsBOOL(AppKit.NSUserInterfaceCompressionOptions.isEmpty)
        # self.assertArgIsBOOL(AppKit.NSUserInterfaceCompressionOptions.setEmpty_)

    @min_sdk_level("10.13")
    def testProtocols(self):
        objc.protocolNamed("NSUserInterfaceCompression")



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSEnumeratorInteraction(TestCase):
    def setUp(self):
        self.arrayContainer = Foundation.NSArray.arrayWithArray_(range(100))

    def testNoFastEnumeration(self):
        self.assertNotHasAttr(Foundation, "NSFastEnumerationState")

    def testInOperator(self):
        y = []
        for x in self.arrayContainer.objectEnumerator():
            y.append(x)

        self.assertEqual(len(y), len(self.arrayContainer))
        for i in range(len(y)):
            self.assertEqual(y[i], self.arrayContainer[i])



import os

import CoreFoundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestCFURLEnumerator(TestCase):
    @min_os_level("10.6")
    def testTypes(self):
        self.assertIsCFType(CoreFoundation.CFURLEnumeratorRef)

    @min_os_level("10.6")
    def testFunctions(self):
        v = CoreFoundation.CFURLEnumeratorGetTypeID()
        self.assertIsInstance(v, int)

        ref = CoreFoundation.CFURLEnumeratorCreateForDirectoryURL(
            None,
            CoreFoundation.CFURLCreateWithFileSystemPath(
                None, os.path.expanduser("~"), CoreFoundation.kCFURLPOSIXPathStyle, True
            ),
            CoreFoundation.kCFURLEnumeratorDefaultBehavior,
            [CoreFoundation.kCFURLNameKey],
        )
        self.assertIsInstance(ref, CoreFoundation.CFURLEnumeratorRef)

        ref = CoreFoundation.CFURLEnumeratorCreateForMountedVolumes(
            None, CoreFoundation.kCFURLEnumeratorDefaultBehavior, None
        )  # [kCFURLNameKey])
        self.assertIsInstance(ref, CoreFoundation.CFURLEnumeratorRef)

        self.assertArgIsOut(CoreFoundation.CFURLEnumeratorGetNextURL, 1)
        self.assertArgIsOut(CoreFoundation.CFURLEnumeratorGetNextURL, 2)
        sts, url, err = CoreFoundation.CFURLEnumeratorGetNextURL(ref, None, None)
        self.assertEqual(sts, CoreFoundation.kCFURLEnumeratorSuccess)
        self.assertIsInstance(url, CoreFoundation.CFURLRef)
        self.assertEqual(err, None)

        lvl = CoreFoundation.CFURLEnumeratorGetDescendentLevel(ref)
        self.assertIsInstance(lvl, int)

        CoreFoundation.CFURLEnumeratorSkipDescendents(ref)

        self.assertResultIsBOOL(CoreFoundation.CFURLEnumeratorGetSourceDidChange)
        v = CoreFoundation.CFURLEnumeratorGetSourceDidChange(ref)
        self.assertIsInstance(v, bool)

    @min_os_level("10.6")
    def testConstants(self):
        self.assertEqual(CoreFoundation.kCFURLEnumeratorDefaultBehavior, 0)
        self.assertEqual(CoreFoundation.kCFURLEnumeratorDescendRecursively, 1 << 0)
        self.assertEqual(CoreFoundation.kCFURLEnumeratorSkipInvisibles, 1 << 1)
        self.assertEqual(
            CoreFoundation.kCFURLEnumeratorGenerateFileReferenceURLs, 1 << 2
        )
        self.assertEqual(CoreFoundation.kCFURLEnumeratorSkipPackageContents, 1 << 3)
        self.assertEqual(
            CoreFoundation.kCFURLEnumeratorIncludeDirectoriesPreOrder, 1 << 4
        )
        self.assertEqual(
            CoreFoundation.kCFURLEnumeratorIncludeDirectoriesPostOrder, 1 << 5
        )
        self.assertEqual(
            CoreFoundation.kCFURLEnumeratorGenerateRelativePathURLs, 1 << 6
        )
        self.assertEqual(CoreFoundation.kCFURLEnumeratorSuccess, 1)
        self.assertEqual(CoreFoundation.kCFURLEnumeratorEnd, 2)
        self.assertEqual(CoreFoundation.kCFURLEnumeratorError, 3)
        self.assertEqual(CoreFoundation.kCFURLEnumeratorDirectoryPostOrderSuccess, 4)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSViewHelper(AppKit.NSObject):
    def view_stringForToolTip_point_userData_(self, a, b, c, d):
        return 1

    def layer_shouldInheritContentsScale_fromWindow_(self, a, b, c):
        return 1


class ObjCTestNSView_KnowPageRange(AppKit.NSView):
    def knowsPageRange_(self, aRange):
        return objc.YES, (1, 10)

    def rectForPage_(self, page):
        return ((1, 1), (2, 2))


class TestNSView(TestCase):
    def test_knowsPageRange(self):
        method = ObjCTestNSView_KnowPageRange.knowsPageRange_
        self.assertEqual(
            method.__metadata__()["arguments"][2]["type"],
            b"o^" + AppKit.NSRange.__typestr__,
        )

    def test_rectForPage(self):
        method = ObjCTestNSView_KnowPageRange.rectForPage_

        self.assertResultHasType(method, AppKit.NSRect.__typestr__)
        self.assertEqual(
            objc.splitSignature(method.signature),
            objc.splitSignature(AppKit.NSRect.__typestr__ + b"@:" + objc._C_NSInteger),
        )


class TestHeader(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSViewNotSizable, 0)
        self.assertEqual(AppKit.NSViewMinXMargin, 1)
        self.assertEqual(AppKit.NSViewWidthSizable, 2)
        self.assertEqual(AppKit.NSViewMaxXMargin, 4)
        self.assertEqual(AppKit.NSViewMinYMargin, 8)
        self.assertEqual(AppKit.NSViewHeightSizable, 16)
        self.assertEqual(AppKit.NSViewMaxYMargin, 32)

        self.assertEqual(AppKit.NSNoBorder, 0)
        self.assertEqual(AppKit.NSLineBorder, 1)
        self.assertEqual(AppKit.NSBezelBorder, 2)
        self.assertEqual(AppKit.NSGrooveBorder, 3)

        self.assertIsInstance(AppKit.NSViewFrameDidChangeNotification, str)
        self.assertIsInstance(AppKit.NSViewFocusDidChangeNotification, str)
        self.assertIsInstance(AppKit.NSViewBoundsDidChangeNotification, str)
        self.assertIsInstance(AppKit.NSViewGlobalFrameDidChangeNotification, str)

    @min_os_level("10.5")
    def testConstants10_5(self):
        self.assertIsInstance(AppKit.NSFullScreenModeAllScreens, str)
        self.assertIsInstance(AppKit.NSFullScreenModeSetting, str)
        self.assertIsInstance(AppKit.NSFullScreenModeWindowLevel, str)
        self.assertIsInstance(AppKit.NSViewDidUpdateTrackingAreasNotification, str)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertIsInstance(
            AppKit.NSFullScreenModeApplicationPresentationOptions, str
        )

        self.assertEqual(AppKit.NSViewLayerContentsRedrawNever, 0)
        self.assertEqual(AppKit.NSViewLayerContentsRedrawOnSetNeedsDisplay, 1)
        self.assertEqual(AppKit.NSViewLayerContentsRedrawDuringViewResize, 2)
        self.assertEqual(AppKit.NSViewLayerContentsRedrawBeforeViewResize, 3)

        self.assertEqual(AppKit.NSViewLayerContentsPlacementScaleAxesIndependently, 0)
        self.assertEqual(AppKit.NSViewLayerContentsPlacementScaleProportionallyToFit, 1)
        self.assertEqual(
            AppKit.NSViewLayerContentsPlacementScaleProportionallyToFill, 2
        )
        self.assertEqual(AppKit.NSViewLayerContentsPlacementCenter, 3)
        self.assertEqual(AppKit.NSViewLayerContentsPlacementTop, 4)
        self.assertEqual(AppKit.NSViewLayerContentsPlacementTopRight, 5)
        self.assertEqual(AppKit.NSViewLayerContentsPlacementRight, 6)
        self.assertEqual(AppKit.NSViewLayerContentsPlacementBottomRight, 7)
        self.assertEqual(AppKit.NSViewLayerContentsPlacementBottom, 8)
        self.assertEqual(AppKit.NSViewLayerContentsPlacementBottomLeft, 9)
        self.assertEqual(AppKit.NSViewLayerContentsPlacementLeft, 10)
        self.assertEqual(AppKit.NSViewLayerContentsPlacementTopLeft, 11)

        self.assertIsInstance(AppKit.NSDefinitionPresentationTypeKey, str)
        self.assertIsInstance(AppKit.NSDefinitionPresentationTypeOverlay, str)
        self.assertIsInstance(
            AppKit.NSDefinitionPresentationTypeDictionaryApplication, str
        )

    @min_os_level("10.9")
    def testConstants10_9(self):
        self.assertEqual(AppKit.NSViewLayerContentsRedrawCrossfade, 4)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSView.isDescendantOf_)
        self.assertResultIsBOOL(AppKit.NSView.isHidden)
        self.assertResultIsBOOL(AppKit.NSView.isHiddenOrHasHiddenAncestor)
        self.assertArgIsBOOL(AppKit.NSView.setHidden_, 0)
        self.assertResultIsBOOL(AppKit.NSView.needsToDrawRect_)
        self.assertResultIsBOOL(AppKit.NSView.wantsDefaultClipping)

        self.assertArgIsFunction(
            AppKit.NSView.sortSubviewsUsingFunction_context_, 0, b"i@@@", False
        )
        self.assertArgHasType(AppKit.NSView.sortSubviewsUsingFunction_context_, 1, b"@")

        self.assertResultIsBOOL(AppKit.NSView.postsFrameChangedNotifications)
        self.assertArgIsBOOL(AppKit.NSView.setPostsFrameChangedNotifications_, 0)
        self.assertResultIsBOOL(AppKit.NSView.autoresizesSubviews)
        self.assertArgIsBOOL(AppKit.NSView.setAutoresizesSubviews_, 0)
        self.assertResultIsBOOL(AppKit.NSView.isFlipped)
        self.assertResultIsBOOL(AppKit.NSView.isRotatedFromBase)
        self.assertResultIsBOOL(AppKit.NSView.isRotatedOrScaledFromBase)
        self.assertResultIsBOOL(AppKit.NSView.isOpaque)
        self.assertResultIsBOOL(AppKit.NSView.canDraw)
        self.assertArgIsBOOL(AppKit.NSView.setNeedsDisplay_, 0)
        self.assertResultIsBOOL(AppKit.NSView.needsDisplay)
        self.assertResultIsBOOL(AppKit.NSView.lockFocusIfCanDraw)
        self.assertResultIsBOOL(AppKit.NSView.lockFocusIfCanDrawInContext_)
        self.assertResultIsBOOL(AppKit.NSView.scrollRectToVisible_)
        self.assertResultIsBOOL(AppKit.NSView.autoscroll_)
        self.assertResultIsBOOL(AppKit.NSView.mouse_inRect_)
        self.assertResultIsBOOL(AppKit.NSView.performKeyEquivalent_)
        self.assertResultIsBOOL(AppKit.NSView.acceptsFirstMouse_)
        self.assertResultIsBOOL(AppKit.NSView.shouldDelayWindowOrderingForEvent_)
        self.assertResultIsBOOL(AppKit.NSView.needsPanelToBecomeKey)
        self.assertResultIsBOOL(AppKit.NSView.mouseDownCanMoveWindow)
        self.assertArgHasType(
            AppKit.NSView.addTrackingRect_owner_userData_assumeInside_, 2, b"^v"
        )
        self.assertArgIsBOOL(
            AppKit.NSView.addTrackingRect_owner_userData_assumeInside_, 3
        )
        self.assertResultIsBOOL(AppKit.NSView.shouldDrawColor)
        self.assertResultIsBOOL(AppKit.NSView.postsBoundsChangedNotifications)
        self.assertArgIsBOOL(AppKit.NSView.setPostsBoundsChangedNotifications_, 0)
        self.assertResultIsBOOL(AppKit.NSView.inLiveResize)
        self.assertResultIsBOOL(AppKit.NSView.preservesContentDuringLiveResize)

        self.assertArgHasType(
            AppKit.NSView.getRectsExposedDuringLiveResize_count_,
            0,
            b"o^" + AppKit.NSRect.__typestr__,
        )
        m = AppKit.NSView.getRectsExposedDuringLiveResize_count_.__metadata__()[
            "arguments"
        ][2]
        self.assertEqual(m["c_array_of_fixed_length"], 4)
        self.assertArgIsOut(AppKit.NSView.getRectsExposedDuringLiveResize_count_, 1)

        self.assertResultIsBOOL(AppKit.NSView.performMnemonic_)
        self.assertResultIsBOOL(AppKit.NSView.canBecomeKeyView)
        self.assertResultIsBOOL(AppKit.NSView.knowsPageRange_)
        self.assertArgIsOut(AppKit.NSView.adjustPageWidthNew_left_right_limit_, 0)
        self.assertArgIsOut(AppKit.NSView.adjustPageHeightNew_top_bottom_limit_, 0)

        self.assertArgIsBOOL(
            AppKit.NSView.dragImage_at_offset_event_pasteboard_source_slideBack_, 6
        )
        self.assertResultIsBOOL(AppKit.NSView.dragFile_fromRect_slideBack_event_)
        self.assertArgIsBOOL(AppKit.NSView.dragFile_fromRect_slideBack_event_, 2)
        self.assertResultIsBOOL(
            AppKit.NSView.dragPromisedFilesOfTypes_fromRect_source_slideBack_event_
        )
        self.assertArgIsBOOL(
            AppKit.NSView.dragPromisedFilesOfTypes_fromRect_source_slideBack_event_, 3
        )

    @min_os_level("10.5")
    def testMethods10_5(self):
        self.assertArgIsBOOL(AppKit.NSView.setWantsLayer_, 0)
        self.assertResultIsBOOL(AppKit.NSView.wantsLayer)
        self.assertResultIsBOOL(AppKit.NSView.enterFullScreenMode_withOptions_)
        self.assertResultIsBOOL(AppKit.NSView.isInFullScreenMode)

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertResultIsBOOL(AppKit.NSView.canDrawConcurrently)
        self.assertArgIsBOOL(AppKit.NSView.setCanDrawConcurrently_, 0)
        self.assertResultIsBOOL(AppKit.NSView.acceptsTouchEvents)
        self.assertArgIsBOOL(AppKit.NSView.setAcceptsTouchEvents_, 0)
        self.assertResultIsBOOL(AppKit.NSView.wantsRestingTouches)
        self.assertArgIsBOOL(AppKit.NSView.setWantsRestingTouches_, 0)

        self.assertArgHasType(
            AppKit.NSView.showDefinitionForAttributedString_atPoint_,
            1,
            AppKit.NSPoint.__typestr__,
        )

        self.assertArgHasType(
            AppKit.NSView.showDefinitionForAttributedString_range_options_baselineOriginProvider_,  # noqa: B950
            1,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgIsBlock(
            AppKit.NSView.showDefinitionForAttributedString_range_options_baselineOriginProvider_,  # noqa: B950
            3,
            AppKit.NSPoint.__typestr__ + AppKit.NSRange.__typestr__,
        )

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBOOL(AppKit.NSView.isDrawingFindIndicator)

        try:
            AppKit.NSView.layer_shouldInheritContentsScale_fromWindow_
        except AttributeError:
            pass
        else:
            self.assertResultIsBOOL(
                AppKit.NSView.layer_shouldInheritContentsScale_fromWindow_
            )

        self.assertResultIsBOOL(
            TestNSViewHelper.layer_shouldInheritContentsScale_fromWindow_
        )

    @min_os_level("10.8")
    def testMethods10_8(self):
        self.assertResultIsBOOL(AppKit.NSView.wantsUpdateLayer)

    @min_os_level("10.9")
    def testMethods10_9(self):
        self.assertResultIsBOOL(AppKit.NSView.canDrawSubviewsIntoLayer)
        self.assertArgIsBOOL(AppKit.NSView.setCanDrawSubviewsIntoLayer_, 0)
        self.assertResultIsBOOL(AppKit.NSView.layerUsesCoreImageFilters)
        self.assertArgIsBOOL(AppKit.NSView.setLayerUsesCoreImageFilters_, 0)
        self.assertResultIsBOOL(AppKit.NSView.isCompatibleWithResponsiveScrolling)

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(AppKit.NSView.allowsVibrancy)

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertResultIsBOOL(AppKit.NSView.wantsExtendedDynamicRangeOpenGLSurface)
        self.assertArgIsBOOL(
            AppKit.NSView.setWantsExtendedDynamicRangeOpenGLSurface_, 0
        )

    def testProtocol(self):
        self.assertArgHasType(
            TestNSViewHelper.view_stringForToolTip_point_userData_,
            2,
            AppKit.NSPoint.__typestr__,
        )
        self.assertArgHasType(
            TestNSViewHelper.view_stringForToolTip_point_userData_, 3, b"^v"
        )

    @min_sdk_level("10.14")
    def testProtocolObjects(self):
        objc.protocolNamed("NSViewLayerContentScaleDelegate")
        objc.protocolNamed("NSViewToolTipOwner")

    def testMissingTests(self):
        v = AppKit.NSView.alloc().init()
        v.setNeedsDisplayInRect_(((0, 0), (50, 50)))
        r = v.getRectsBeingDrawn_count_(None, None)
        self.assertIsInstance(r, tuple)
        self.assertEqual(len(r), 2)
        self.assertIsInstance(r[0], tuple)
        self.assertIsInstance(r[1], int)
        if r[1] != 0:
            self.assertIsInstance(r[0][0], AppKit.NSRect)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSTokenFieldCellHelper(AppKit.NSObject):
    def tokenFieldCell_completionsForSubstring_indexOfToken_indexOfSelectedItem_(
        self, a, b, c, d
    ):
        return 1

    def tokenFieldCell_shouldAddObjects_atIndex_(self, a, b, c):
        return 1

    def tokenFieldCell_writeRepresentedObjects_toPasteboard_(self, a, b, c):
        return 1

    def tokenFieldCell_hasMenuForRepresentedObject_(self, a, b):
        return 1

    def tokenFieldCell_styleForRepresentedObject_(self, a, b):
        return 1


class TestNSTokenFieldCell(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSDefaultTokenStyle, 0)
        self.assertEqual(AppKit.NSTokenStyleNone, 1)
        self.assertEqual(AppKit.NSTokenStyleRounded, 2)

        self.assertEqual(AppKit.NSPlainTextTokenStyle, 1)  # Deprecated
        self.assertEqual(AppKit.NSRoundedTokenStyle, 2)  # Deprecated

        self.assertEqual(AppKit.NSTokenStyleDefault, 0)
        self.assertEqual(AppKit.NSTokenStyleNone, 1)
        self.assertEqual(AppKit.NSTokenStyleRounded, 2)
        self.assertEqual(AppKit.NSTokenStyleSquared, 3)
        self.assertEqual(AppKit.NSTokenStylePlainSquared, 4)

    @min_os_level("10.10")
    def testContants10_10(self):
        self.assertEqual(AppKit.NSTokenStyleSquared, 3)
        self.assertEqual(AppKit.NSTokenStylePlainSquared, 4)

    @min_sdk_level("10.7")
    def testProtocolObjects(self):
        objc.protocolNamed("NSTokenFieldCellDelegate")

    def testProtocols(self):
        self.assertArgHasType(
            TestNSTokenFieldCellHelper.tokenFieldCell_completionsForSubstring_indexOfToken_indexOfSelectedItem_,  # noqa: B950
            2,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSTokenFieldCellHelper.tokenFieldCell_completionsForSubstring_indexOfToken_indexOfSelectedItem_,  # noqa: B950
            3,
            b"o^" + objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSTokenFieldCellHelper.tokenFieldCell_shouldAddObjects_atIndex_,
            2,
            objc._C_NSUInteger,
        )
        self.assertResultIsBOOL(
            TestNSTokenFieldCellHelper.tokenFieldCell_writeRepresentedObjects_toPasteboard_
        )
        self.assertResultIsBOOL(
            TestNSTokenFieldCellHelper.tokenFieldCell_hasMenuForRepresentedObject_
        )
        self.assertResultHasType(
            TestNSTokenFieldCellHelper.tokenFieldCell_styleForRepresentedObject_,
            objc._C_NSUInteger,
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSCollectionViewHelper(AppKit.NSObject):
    def collectionView_acceptDrop_indexPath_dropOperation_(self, c, d, p, o):
        return 1

    def collectionView_validateDrop_proposedIndexPath_dropOperation_(self, c, d, p, o):
        return 1

    def collectionView_draggingImageForItemsAtIndexPaths_withEvent_offset_(
        self, v, p, e, o
    ):
        return 1

    def collectionView_writeItemsAtIndexPaths_toPasteboard_(self, v, p, p2):
        return True

    def collectionView_canDragItemsAtIndexPaths_withEvent_(self, c, p, e):
        return True

    def numberOfSectionsInCollectionView_(self, v):
        return 1

    def collectionView_numberOfItemsInSection_(self, v, s):
        return 1

    def collectionView_canDragItemsAtIndexes_withEvent_(self, v, i, e):
        return 1

    def collectionView_writeItemsAtIndexes_toPasteboard_(self, v, i, p):
        return 1

    def collectionView_draggingImageForItemsAtIndexes_withEvent_offset_(
        self, v, i, e, o
    ):
        return 1

    def collectionView_validateDrop_proposedIndex_dropOperation_(self, v, d, i, o):
        return 1

    def collectionView_acceptDrop_index_dropOperation_(self, v, d, i, o):
        return 1

    def collectionView_pasteboardWriterForItemAtIndex_(self, v, i):
        pass

    def collectionView_draggingSession_willBeginAtPoint_forItemsAtIndexes_(
        self, v, i, j, p
    ):
        pass

    def collectionView_draggingSession_willBeginAtPoint_forItemsAtIndexPaths_(
        self, v, i, j, p
    ):
        pass

    def collectionView_draggingSession_endedAtPoint_draggingOperation_(
        self, a, b, c, d
    ):
        pass

    def collectionView_shouldChangeItemsAtIndexPaths_toHighlightState_(self, a, b, c):
        pass

    def collectionView_didChangeItemsAtIndexPaths_toHighlightState_(self, a, b, c):
        pass


class TestNSCollectionView(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSCollectionViewItem.isSelected)
        self.assertArgIsBOOL(AppKit.NSCollectionViewItem.setSelected_, 0)

        self.assertResultIsBOOL(AppKit.NSCollectionView.isFirstResponder)

        self.assertResultIsBOOL(AppKit.NSCollectionView.isSelectable)
        self.assertArgIsBOOL(AppKit.NSCollectionView.setSelectable_, 0)
        self.assertResultIsBOOL(AppKit.NSCollectionView.allowsMultipleSelection)
        self.assertArgIsBOOL(AppKit.NSCollectionView.setAllowsMultipleSelection_, 0)

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertResultHasType(
            AppKit.NSCollectionView.frameForItemAtIndex_, AppKit.NSRect.__typestr__
        )
        self.assertArgIsBOOL(
            AppKit.NSCollectionView.setDraggingSourceOperationMask_forLocal_, 1
        )
        self.assertArgHasType(
            AppKit.NSCollectionView.draggingImageForItemsAtIndexes_withEvent_offset_,
            2,
            b"N^" + AppKit.NSPoint.__typestr__,
        )

        self.assertResultIsBOOL(
            TestNSCollectionViewHelper.collectionView_canDragItemsAtIndexes_withEvent_
        )
        self.assertResultIsBOOL(
            TestNSCollectionViewHelper.collectionView_writeItemsAtIndexes_toPasteboard_
        )
        self.assertArgHasType(
            TestNSCollectionViewHelper.collectionView_draggingImageForItemsAtIndexes_withEvent_offset_,  # noqa: B950
            3,
            b"N^" + AppKit.NSPoint.__typestr__,
        )

        self.assertResultHasType(
            TestNSCollectionViewHelper.collectionView_validateDrop_proposedIndex_dropOperation_,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSCollectionViewHelper.collectionView_validateDrop_proposedIndex_dropOperation_,
            2,
            b"N^" + objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSCollectionViewHelper.collectionView_validateDrop_proposedIndex_dropOperation_,
            3,
            b"N^" + objc._C_NSInteger,
        )

        self.assertResultIsBOOL(
            TestNSCollectionViewHelper.collectionView_acceptDrop_index_dropOperation_
        )
        self.assertArgHasType(
            TestNSCollectionViewHelper.collectionView_acceptDrop_index_dropOperation_,
            2,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSCollectionViewHelper.collectionView_acceptDrop_index_dropOperation_,
            3,
            objc._C_NSInteger,
        )

        self.assertArgHasType(
            TestNSCollectionViewHelper.collectionView_pasteboardWriterForItemAtIndex_,
            1,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSCollectionViewHelper.collectionView_draggingSession_willBeginAtPoint_forItemsAtIndexes_,  # noqa: B950
            2,
            AppKit.NSPoint.__typestr__,
        )
        self.assertArgHasType(
            TestNSCollectionViewHelper.collectionView_draggingSession_endedAtPoint_draggingOperation_,  # noqa: B950
            2,
            AppKit.NSPoint.__typestr__,
        )
        self.assertArgHasType(
            TestNSCollectionViewHelper.collectionView_draggingSession_endedAtPoint_draggingOperation_,  # noqa: B950
            3,
            objc._C_NSInteger,
        )

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertResultIsBOOL(AppKit.NSCollectionView.allowsEmptySelection)
        self.assertArgIsBOOL(AppKit.NSCollectionView.setAllowsEmptySelection_, 0)

        self.assertArgIsBlock(
            AppKit.NSCollectionView.performBatchUpdates_completionHandler_, 0, b"v"
        )
        self.assertArgIsBlock(
            AppKit.NSCollectionView.performBatchUpdates_completionHandler_, 1, b"vZ"
        )

        self.assertArgIsInOut(
            AppKit.NSCollectionView.draggingImageForItemsAtIndexPaths_withEvent_offset_,
            2,
        )

        self.assertArgIsBlock(
            AppKit.NSSet.enumerateIndexPathsWithOptions_usingBlock_, 1, b"v@o^Z"
        )

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertResultIsBOOL(
            AppKit.NSCollectionView.backgroundViewScrollsWithContent
        )
        self.assertArgIsBOOL(
            AppKit.NSCollectionView.setBackgroundViewScrollsWithContent_, 0
        )

    def testConstants(self):
        self.assertEqual(AppKit.NSCollectionViewScrollPositionNone, 0)
        self.assertEqual(AppKit.NSCollectionViewScrollPositionTop, 1 << 0)
        self.assertEqual(
            AppKit.NSCollectionViewScrollPositionCenteredVertically, 1 << 1
        )
        self.assertEqual(AppKit.NSCollectionViewScrollPositionBottom, 1 << 2)
        self.assertEqual(
            AppKit.NSCollectionViewScrollPositionNearestHorizontalEdge, 1 << 9
        )
        self.assertEqual(AppKit.NSCollectionViewScrollPositionLeft, 1 << 3)
        self.assertEqual(
            AppKit.NSCollectionViewScrollPositionCenteredHorizontally, 1 << 4
        )
        self.assertEqual(AppKit.NSCollectionViewScrollPositionRight, 1 << 5)
        self.assertEqual(AppKit.NSCollectionViewScrollPositionLeadingEdge, 1 << 6)
        self.assertEqual(AppKit.NSCollectionViewScrollPositionTrailingEdge, 1 << 7)
        self.assertEqual(
            AppKit.NSCollectionViewScrollPositionNearestVerticalEdge, 1 << 8
        )

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(AppKit.NSCollectionViewDropOn, 0)
        self.assertEqual(AppKit.NSCollectionViewDropBefore, 1)

    @min_os_level("10.11")
    def testConstants10_11(self):
        self.assertEqual(AppKit.NSCollectionViewItemHighlightNone, 0)
        self.assertEqual(AppKit.NSCollectionViewItemHighlightForSelection, 1)
        self.assertEqual(AppKit.NSCollectionViewItemHighlightForDeselection, 2)
        self.assertEqual(AppKit.NSCollectionViewItemHighlightAsDropTarget, 3)

    @min_sdk_level("10.10")
    def testProtocols(self):
        objc.protocolNamed("NSCollectionViewDelegate")

    @min_sdk_level("10.11")
    def testProtocols10_11(self):
        objc.protocolNamed("NSCollectionViewElement")

        objc.protocolNamed("NSCollectionViewDataSource")
        self.assertResultHasType(
            TestNSCollectionViewHelper.collectionView_numberOfItemsInSection_,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSCollectionViewHelper.collectionView_numberOfItemsInSection_,
            1,
            objc._C_NSInteger,
        )

        self.assertResultHasType(
            TestNSCollectionViewHelper.numberOfSectionsInCollectionView_,
            objc._C_NSInteger,
        )

        objc.protocolNamed("NSCollectionViewDelegate")
        self.assertResultIsBOOL(
            TestNSCollectionViewHelper.collectionView_canDragItemsAtIndexPaths_withEvent_
        )
        self.assertResultIsBOOL(
            TestNSCollectionViewHelper.collectionView_writeItemsAtIndexPaths_toPasteboard_
        )
        self.assertArgHasType(
            TestNSCollectionViewHelper.collectionView_draggingImageForItemsAtIndexPaths_withEvent_offset_,  # noqa: B950
            3,
            b"N^" + AppKit.NSPoint.__typestr__,
        )

        self.assertArgHasType(
            TestNSCollectionViewHelper.collectionView_validateDrop_proposedIndexPath_dropOperation_,  # noqa: B950
            2,
            b"N^@",
        )
        self.assertResultHasType(
            TestNSCollectionViewHelper.collectionView_validateDrop_proposedIndexPath_dropOperation_,  # noqa: B950
            objc._C_NSUInteger,
        )

        self.assertResultIsBOOL(
            TestNSCollectionViewHelper.collectionView_acceptDrop_indexPath_dropOperation_
        )
        self.assertArgHasType(
            TestNSCollectionViewHelper.collectionView_acceptDrop_indexPath_dropOperation_,
            3,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSCollectionViewHelper.collectionView_draggingSession_willBeginAtPoint_forItemsAtIndexPaths_,  # noqa: B950
            2,
            AppKit.NSPoint.__typestr__,
        )

        self.assertArgHasType(
            TestNSCollectionViewHelper.collectionView_shouldChangeItemsAtIndexPaths_toHighlightState_,  # noqa: B950
            2,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSCollectionViewHelper.collectionView_didChangeItemsAtIndexPaths_toHighlightState_,  # noqa: B950
            2,
            objc._C_NSInteger,
        )

    @min_sdk_level("10.12")
    def testProtocol10_12(self):
        objc.protocolNamed("NSCollectionViewSectionHeaderView")

    @min_sdk_level("10.13")
    def testProtocol10_13(self):
        objc.protocolNamed("NSCollectionViewPrefetching")
